
/* functions fmetapop of updating per column (per year) */ 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <limits.h>
#include <nrutil.h>
#include <nr.h>


#include  "fmetapop.h" 
#include  "MC_patches.h" 
extern FILE *out, *par, *dat, *mon;
extern long seed;
extern int Stochastic_EM, Use_J_model;
extern double logEPS, log1_EPS;



double urandread(long*seed);

#if CHECK
#define urandread(plong) urandread0(plong)          /* uniform numbers read  from file */
#else
#define urandread(plong) ran1(plong)          /* uniform(0,1) */
#endif

double urandread0(long*seed)
{
 double p;
    	fscanf(dat,  " %lf " ,&p);
			return p;

}

# if QUICK == 2  || QUICK == 3

double MC_patches(long nsubiter, int nyear, int npatch, \
 state_t **Xs, int Xs_tot[], int L, int StartPattern, int **ti_missinglist, double **S, double **ADa,\
 double logA[], double **logE1_E, double **ptheta, int npar, double **X, int *nxpar, double **logtrprob,\
 double *pEloglik,double *pE2loglik, int startyear, int endyear,
 int propose_t, int propose_t1, double *accept_p, double **pab)
/* Use Metropolis-Hasting sampling to update all missing patches
  from pretime -L to last data year nyear, updating per year (i.e. per row in data)

   state_t  **Xs;   states with front times, data and imputed missing data 
	 int Xs_tot[] times-vector with total number of occupied patches
	                 	in the non-missing data
   double *logA,    vector of logAreas  
	        **S,      times x patches matrix of connectivities
					**ADA,    patches x patches matrix of distances with elements A[j]^b*exp(-D[i][j]/alpha)   
          **X,      patches x nvars matrix of extra explanatory variables  
          **ptheta, pointers to rows of the times x npar matrix of parameter values  
					**pab     see main program fmetapop  (not used yet)
The double ** pointers involving times have fist index running from  -(L+3) to nyears
 in their definition in the mainprogram

 if !regional stochast 2-vector logE1_E is used to once calculate for all t
	  logE1_E[i][0] = log(E_i) and  logE1_E[i][1] = log(1-E_i)

Returns the log likelihood of last iteration 
Eloglik is the average loglik
E2loglik is the estimated variance of the average likelihood, var(loglik)/n ,
		 with n the number of evaluations

accept_p = prob of acceptance of proposal-vectors for a year

Note:
	 **logtrprob is a times x npatch matrix with log of transition probabilities,
	 
startyear and endyear determine the year over which the likelihood is calculated:

   if (startyear == 1 and endyear == nyear ) { usage of turnover information only	 }
   if (startyear == -L+1 and endyear == nyear ) { usage of spatial + turnover information 	 }
	  
*/
{
 int i, j, i1, k, iterS, ibeginmissing, t, n_update =0;
 long iter, iterEM=0;
 state_t stot;
 double r, logE, logC, Si, dif_S, lp,
   accept_prob_Xs = -1., accept_se_Xs, accept_Xs, loglik,
	 Eloglik =0, /* average log likelihood if Stochastic EM*/
	 E2loglik =0;  /* ss of log likelihood if Stochastic EM*/
 double *logJxp, *J;
 # if CHECK
 double fcheck,dum, p_1, matlablik;
 int xm;
 # endif

 #if !REGIONAL_STOCHAST
  for (i=1;i<=npatch ;i++ ) {
		lp = 0.0;
		for (j= 1;j<= nxpar[0] ;j++ )  lp +=  ptheta[0][npar - nxpar[0] - nxpar[1] +j]*X[i][j];
		logE = DMIN(ptheta[0][1] - ptheta[0][2] * logA[i] + lp, log1_EPS); 
	  /* logE = DMIN(ptheta[0][1] - ptheta[0][2] * logA[i], log1_EPS); */
    logE1_E[i][0] = logE = DMAX(logE, logEPS);
		logE1_E[i][1]= log(1.0 - exp(logE)); /* log(1-E) */

	}
 #endif

 /*	*/



 logJxp = S[propose_t1-1];	/*workarry in propose_col; this array of S is not used further*/
 J = logtrprob[propose_t1-1];	/*workarray */

  

/*
		fprintfdmatrix(mon, logtrprob, nyear-2, nyear, 1, npatch, " %10.3lf " );
 		fprintfimatrix(mon, Xs, nyear-2, nyear, 1, npatch, " %3d " );
 		fprintfdmatrix(mon, S, nyear-2, nyear-1, 1, npatch, " %10.3lf " );
*/


/*

		fprintf(mon," start  = %d \n", Xs[2][7]);
		fprintfdmatrix(mon, logtrprob, 2, 3, 1, npatch, " %10.3lf " );
		fprintfdmatrix(mon, S, 2, 2, 1, npatch, " %10.3lf " );
*/

	for (iter = 1; iter<=nsubiter ; iter++ ) {
		
			if (StartPattern >=0) {
	 /* update patches at pretime -L using the J-based connectivity
	    i.e. the connectivity weighted by the Hanski's incidence probability,
			calculated iteratively */
		  t = -L;
			iterS = 0;
			do{
  			 dif_S = 0.0;
				 iterS++;
			 	 for (i=1;i<=npatch ;i++ ) {
				   logE = logptrans( 0, 1, S[t][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
				   logC = logptrans( 1, 0, S[t][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
					 /* log J */
					 J[i] = 1./(1. + exp(logE - logC));
				 }

			  for ( i=1;i<=npatch ;i++ ) {
			  	Si = 0.0;
			  	for ( j=1;j<=npatch ;j++ ) Si += J[j] * ADa[i][j];
					/* avoid problems with S = 0 in first iteration */
					if (iterS>1) {if (fabs(log((Si+EPS)/(S[t][i]+EPS))) > dif_S );dif_S = fabs(log(Si/S[t][i]));}
			  	S[t][i] = Si;	
			  }
		  } while ( (dif_S >= 0.0001 || iterS < 2) && iterS <=100 );
		  for (i=1;i<=npatch ;i++ ){
					/* backward, BUT with ptheta and expected iterated S from year -L*/
		        logtrprob[t][i] = logptrans( 1, Xs[t+1][i], S[t][i], logA[i], logE1_E[i], ptheta[-L], npar, X[i], nxpar);
	    }

		    do {
		      stot = 0;
		   	  for (i=1;i<=npatch ;i++ ){
		        Xs[t][i] = (logtrprob[t][i]>log(urand(&seed)));
		        stot +=  Xs[t][i];
		       }
		    } while (stot < 1);

		  /*calculate connectivity for time t = -L and log transition probs for -L+1*/
		    calc_S2( S[-L], npatch , Xs[-L], ADa);
        calc_logtrprob(npatch, -L+1, -L+1, Xs, S, logA, logE1_E, ptheta, npar, X, nxpar, logtrprob);
 			} /* if StartPattern */


	 /* update patches at pretimes from 0 to -L+1 */

	 for (t=0; t>-L; t--){

   #if QUICK == 2 
		 /* proposal generated on the basis of S[t]  */
		 r = propose_column(npatch, Xs[t],  Xs[propose_t], Xs[t+1],
		    logtrprob[t], logtrprob[t+1], logtrprob[propose_t], logtrprob[propose_t1],
		    S[t], S[propose_t], ADa, logA, logE1_E, ptheta[t+1],  npar,  X,  nxpar, logJxp);

	 #elif  QUICK == 3 
		 /* proposal generated on the basis of S[t-1]  */
		 r= propose_column0(npatch, Xs[t-1], Xs[t],  Xs[propose_t], Xs[t+1],
		    logtrprob[t], logtrprob[t+1], logtrprob[propose_t], logtrprob[propose_t1],
		    S[t-1], S[propose_t], ADa,  logA, logE1_E, ptheta[t+1],  npar,  X,  nxpar, logJxp);
	
   #endif
	 
	 # if CHECK 
	     for (i1=1; i1<=npatch;i1++ ) {
			  fscanf(par,  " %lf %d " ,&p_1, &xm);
					 fprintf(mon," STATE  i = %3d p_1=	 %20.17lf Xs[propose_t] = %3d mX= %3d \n", i1, p_1, Xs[propose_t][i1], xm );
				   if (xm!= Xs[propose_t][i1]){
					 fprintf(mon," DIFF_STATE i = %3d Xs[propose_t] = %3d\n", i1, Xs[propose_t][i1] );
					 }
			 }
			 
	     for (i1=1; i1<=2*npatch;i1++ ) fscanf(par,  " %lf " ,&fcheck);
     	 fscanf(par,  " %lf " ,&fcheck);
			 fprintf(mon," rdiff = %20.17lf r= %20.17lf  fcheck =%20.17lf\n", 
				  r - fcheck, r, fcheck);
				fcheck-= r;
				if (fabs(fcheck)>1.E-10) {
				  fprintf(mon," LARGE DIFF_F in iter = %10d \n", iter);
					}
	 # endif

		 if ( r > urandread(&seed) ) {
		 /* accept proposal row for year t		 */
			 accept_Xs = 1;
		   swap_pi(&(Xs[propose_t]),&(Xs[t]));
		   swap_pd(&(S[propose_t]),&(S[t]));
		   swap_pd(&(logtrprob[propose_t]), &(logtrprob[t]));
		   swap_pd(&(logtrprob[propose_t1]),&(logtrprob[t+1]));
			 
		 }
		 else {
		   accept_Xs = 0;
		 }
		 n_update++;
		 update_mean_sumss(n_update, accept_Xs, &accept_prob_Xs, &accept_se_Xs);

	 }


	/* update missings in all data years except last */

	  ibeginmissing = 1;
/*	t = ti_missinglist[ibeginmissing][1]; */
		while ((t=ti_missinglist[ibeginmissing][1]) < nyear) {

			 k = ibeginmissing;

			 if (ti_missinglist[k][1] != ti_missinglist[k+1][1]) {
			 /*one missing patch in a year only, hence do Gibbs */

			 i1 = ti_missinglist[k][2];
			 Gibbs_one_patch(npatch, Xs[t], Xs[propose_t], Xs[t+1],
         logtrprob[t], &(logtrprob[t+1]),  &(logtrprob[propose_t1]),
			    &(S[t]), &(S[propose_t]), logA, logE1_E, ADa, ptheta[t+1],  npar,  X,  nxpar, 
					&k, ti_missinglist);
			 ibeginmissing = k;
			 }
			 else {
			 
			 /*more than one missing patch in a year, hence do Metropolis-Hasting */

   #if QUICK == 2 
		 /* proposal generated on the basis of S[t]  */
	  			 r = propose_column_data( npatch, Xs[t],  Xs[propose_t], Xs[t+1],   
			    logtrprob[t], logtrprob[t+1], logtrprob[propose_t], logtrprob[propose_t1],
			    S[t], S[propose_t], ADa, logA, logE1_E, ptheta[t+1],  npar,  X,  nxpar, 
					&k, ti_missinglist, Xs_tot[t], logJxp);

	 #elif  QUICK == 3 
		 /* proposal generated on the basis of S[t-1]  */
 			 r = propose_column0_data( npatch, Xs[t-1], Xs[t],  Xs[propose_t], Xs[t+1],   
			    logtrprob[t], logtrprob[t+1],  logtrprob[propose_t], logtrprob[propose_t1],
			    S[t-1], S[propose_t], ADa,  logA, logE1_E, ptheta[t+1],  npar,  X,  nxpar, 
					&k, ti_missinglist, Xs_tot[t], logJxp);

   #endif 

	 # if CHECK 
	     for (i1=1; i1<=npatch;i1++ ) {
			  fscanf(par,  " %lf %d " ,&p_1, &xm);
					 fprintf(mon," STATE  i = %3d p_1=	 %20.17lf Xs[propose_t] = %3d mX= %3d \n", i1, p_1, Xs[propose_t][i1], xm );
				   if (xm!= Xs[propose_t][i1]){
					 fprintf(mon," DIFF_STATE i = %3d Xs[propose_t] = %3d\n", i1, Xs[propose_t][i1] );
					 }
			 }
			 
	     for (i1=1; i1<=2*npatch;i1++ ) fscanf(par,  " %lf " ,&fcheck);
     	 fscanf(par,  " %lf " ,&fcheck);
			 fprintf(mon," rdiff = %20.17lf r= %20.17lf  fcheck =%20.17lf\n", 
				  r - fcheck, r, fcheck);
				fcheck-= r;
				if (fabs(fcheck)>1.E-10) {
				  fprintf(mon," LARGE DIFF_F in iter = %10d \n", iter);
					}
	 # endif

  	 if ( r > urandread(&seed) ) {
			 /* accept proposal row for year t		 */
				 accept_Xs = 1;
         swap_pi(&(Xs[propose_t]),&(Xs[t]));
  		   swap_pd(&(S[propose_t]),&(S[t]));
			   swap_pd(&(logtrprob[propose_t1]),&(logtrprob[t+1]));
/*	 Note: non-missing patches in logtrprob[propose_t] are not filled
     Hence, the vector cannot be swapped, hence explicit copy

			   swap_pd(logtrprob[propose_t], logtrprob[t]);
*/	
  	     for ( k = ibeginmissing; ti_missinglist[k][1] == t; k++){
			     i = ti_missinglist[k][2];
					 logtrprob[t][i] = logtrprob[propose_t][i];
				 }


			 }
			 else {
			   accept_Xs = 0;
			 }
			 ibeginmissing = k;
			 n_update++;
			 update_mean_sumss(n_update, accept_Xs, &accept_prob_Xs, &accept_se_Xs);
			 }

		}

	/* update missings in last data year */

	  if ((t=ti_missinglist[ibeginmissing][1]) == nyear)  
		      Gibbs_column_last_t(Xs[t], Xs[propose_t],   
			    logtrprob[t],	&ibeginmissing, ti_missinglist, Xs_tot[t]);
			# if CHECK
      	fscanf(dat,  " %lf " ,&dum);
			# endif
								 
/*	
		 		fprintfdmatrix(mon, logtrprob, nyear-2, nyear, 1, npatch, " %10.3lf " );
 		fprintfimatrix(mon, Xs, nyear-2, nyear, 1, npatch, " %3d " );
 		fprintfdmatrix(mon, S, nyear-2, nyear-1, 1, npatch, " %10.3lf " );

*/


		if (Stochastic_EM !=0) {
			/* calculate average loglikelihood Eloglik, after 10% of iterations */
			if (iter > 0.0 * nsubiter) {
	  		 iterEM++;
			   if (Use_J_model) {
				   	/* Use Hanski' s incidence model for time 1 
					   save the logtransitions for time 1  and set J in column one*/
					   swap_pd(&(logtrprob[propose_t]),&(logtrprob[1]));
						 for (i=1;i<=npatch ;i++ ) {
						   logE = logptrans( 0, 1, S[1][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
						   logC = logptrans( 1, 0, S[1][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
							 if (Xs[1][i]) {
							 /* log J */
							   logtrprob[1][i] = -log(1. + exp(logE - logC));
							 }
							 else {
							 /* log (1-J) */
							   logtrprob[1][i] = -log(1. + exp(logC - logE));
							 }
						 }
				 } /* end Use_J-model */
				 loglik = floglik0(npatch,startyear, endyear, logtrprob);
         if (Use_J_model) swap_pd(&(logtrprob[propose_t]),&(logtrprob[1]));
				 update_mean_sumss(iterEM, loglik, &Eloglik, &E2loglik);
			 	 # if DEBUG==112
				 if ((iter+9)%10==0) fprintf(mon, "%5d %7.3lf\n ",iter, loglik);
				 # endif
 
 			}

		}

		# if CHECK
    fscanf(par,  " %lf " ,&matlablik);

		fprintf(mon," loglikdiff = %20.17lf fpop= %20.17lf  matlab =%20.17lf\n", 
		  loglik - matlablik, loglik, matlablik);
		matlablik -= loglik;
		if (fabs(matlablik)>1.E-10) {
		    fprintf(mon," LARGE DIFF_LIKELIHOOD \n");
				}

		# endif

/*

		fprintf(mon," changes in state = %d %10.3lf; \n", Xs[2][7], loglik);

		fprintfdmatrix(mon, logtrprob, 2, 3, 1, npatch, " %10.3lf " );
		fprintfdmatrix(mon, S, 2, 2, 1, npatch, " %10.3lf " );
*/

	} /*  end iter loop*/

 		/*calc_logtrprob(npatch, -L+1, 0, Xs, S, logA, logE1_E, ptheta, npar, X, nxpar, logtrprob);*/

		# if DEBUG == 103

		fprintf(out," changes in state = %d \n", state_change);
		fprintf(out," logtrprob in Gibb2\n", state_change);
		fprintfdmatrix(out, logtrprob, -L+1, nyear, 1, npatch, " %10.3lf " );
		# endif


    if (Use_J_model) {
		   /* Use Hanski' s incidence model for time 1 
		   save the logtransitions for time 1  and set J in column one*/
		   swap_pd(&(logtrprob[propose_t]),&(logtrprob[1]));
			 for (i=1;i<=npatch ;i++ ) {
			   logE = logptrans( 0, 1, S[1][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
			   logC = logptrans( 1, 0, S[1][i], logA[i], logE1_E[i], ptheta[0], npar, X[i], nxpar);
				 if (Xs[1][i]) {
				 /* log J */
				   logtrprob[1][i] = -log(1. + exp(logE - logC));
				 }
				 else {
				 /* log (1-J) */
				   logtrprob[1][i] = -log(1. + exp(logC - logE));
				 }
			 }
		} /* end Use_J-model */

    loglik = floglik0(npatch, startyear, endyear, logtrprob);

    if (Use_J_model) swap_pd(&(logtrprob[propose_t]),&(logtrprob[1]));
		


 		if (Stochastic_EM && iterEM>1 )	E2loglik /=(iterEM-1);
		
		*pEloglik =  Eloglik;
		*pE2loglik = E2loglik;
    *accept_p = accept_prob_Xs;


	return loglik;
}
#if QUICK == 2
double propose_column(int npatch,
   state_t x1[], state_t x1p[], state_t x2[], 
	 double logtrprobx0_x1[], double logtrprobx1_x2[],
	 double logtrprobx0_x1p[], double logtrprobx1p_x2[],
   double S1[], double S1p[], double** ADa, double logA[], double** logE1_E,\
   double theta2[], int npar, double** X, int *nxpar, double logJxp[])
/*  Generates a proposal column x1p (not for first or last column ) and
    Calculates the jump probability J(x1p| x1) 
		    from the current column x1 of patches at time 1
          to proposed column x1p of patches at time 1
                                  given states at time 0 and time 2
       and the reverse jump probability J(x1|x1p)
		Calculates the ratio of densities and	 															  
    returns the logr = log(ratio of importance ratios) of Metropolis Hasting 
		which is used to accept proposal x1p with prob(min(1,r))  

*/
/* 
 npatch = number of patches
 x0[]     = state of patches at time 0
 x1[]     = state of patches at time 1
 x1p[]  = proposed states at time 1
 x2[]   = states at time 2
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 S1[]   = vector of connectivities of all patches from (current) states for time 2
 S1p[]   = vector of connectivities calculated from x1p, the vector of proposed states
 theta2[] = parameters to calculate probs for time 2

 logJxp = workarray of length npatch containing log(J(xp|x))


INPUT:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1)
 logtrprobx1_x2[] = log transition probabilities for time 2
   (for transition of x1 to x2 time 1 to 2)

OUTPUT: logtrprobx0_x1p[] = log transition probabilities for x0 to x1p
				 (first calculated and used as log transition probabilities for x0 to 1-x1)
        logtrprobx1p_x2 =  log transition probabilities for x1p to x2

*/
{
  int i, stot;
  double f, logr, probi, logJxi, logprobx0_x1pp, logprobx1pp_x2;
	state_t occupied =1;

	   for ( i = 1; i <= npatch; i++){
       logtrprobx0_x1p[i] = log(1-exp(logtrprobx0_x1[i])); /* probs for states x1p = 1-x1 */
		   logtrprobx1p_x2[i] = logptrans(x2[i], occupied-x1[i], S1[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
		 }

		 do{
		 stot = 0;
	   for ( i = 1; i <= npatch; i++){

/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S1 is kept constant during the proposal generation
 			 f = prob("x1p")/prob(x1) = prob(1-x1)/prob(x1) =
			  ( (1-probx0_x1) * probx1p_x2) /	 (probx0_x1 * probx1_x2 )
  From this calculate P(x1p=1|x1)  and logJxp
*/
			 f = exp( logtrprobx0_x1p[i] + logtrprobx1p_x2[i] -
			 			         (logtrprobx0_x1[i]  + logtrprobx1_x2[i]) );
		   if (x1[i]) {
 /* x1switched = "x1p" = 1-x1 = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1p" = 1-x1 = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
       x1p[i] = (probi > urandread(&seed)  );
			 if (x1p[i]) {
			   logJxp[i] = log(probi);
  			 stot++;
			 }
			 else {
			   logJxp[i] = log(1 - probi);
			 }
	   }
 	 } while (stot == 0);

 	 logr = 0.0;
	 calc_S2( S1p, npatch, x1p, ADa);

   for ( i=1; i<=npatch; i++){

 	    logtrprobx1p_x2[i] = logptrans(x2[i], x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);		    


/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S1p is kept constant.
	This oddsratio is needed to calculate the jump probability J(x1|x1p) 
	 			 f = prob("x1pp"=1-x1p)/prob(x1p) =prob(1-x1p)/prob(x1p) =
			  ( (1-probx0_x1p) * probx1pp_x2) /	 (probx0_x1p * probx1p_x2 )
  From this calculate J(x1=1|x1p) and logJxi
*/
	 	  if (x1[i] == x1p[i]) {
  			 logprobx0_x1pp = logtrprobx0_x1p[i];
  			 logtrprobx0_x1p[i] = logtrprobx0_x1[i];
			}
			else {
  			 logprobx0_x1pp = logtrprobx0_x1[i];
			}
/*		logprobx0_x1pp = log(1-exp(logtrprobx0_x1p[i]));*/ /* probs for states x1pp = 1-x1p */
			logprobx1pp_x2 = logptrans(x2[i], occupied-x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
		  f = exp( logprobx0_x1pp  + logprobx1pp_x2 -
			 			         (logtrprobx0_x1p[i] + logtrprobx1p_x2[i]) );

		 if (x1p[i]) {
 /* x1switched = "x1pp" = 1-x1p = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1pp" = 1-x1p = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
			 if (x1[i]) {
			  logJxi = log(probi);
			 }
			 else {
			  logJxi = log(1-probi);
	 	   }
	 		 logr += (logtrprobx0_x1p[i] + logtrprobx1p_x2[i] - logJxp[i]) -
				           (logtrprobx0_x1[i] + logtrprobx1_x2[i] - logJxi);                      ;
	
	   }



   return exp(logr);
}



double propose_column_data(int npatch, state_t x1[], state_t x1p[], state_t x2[],
	 double logtrprobx0_x1[], double logtrprobx1_x2[],
	 double logtrprobx0_x1p[],  double logtrprobx1p_x2[],
   double S1[], double S1p[], double** ADa, double logA[], double** logE1_E, \
   double theta2[], int npar, double** X, int *nxpar,
	 int *ibeginmissing, int **ti_missinglist, int stot_1, double logJxp[])
/*  Generates a proposal column x1p for missings in the data (not for first or last column ) 
    See for further explanation propose-column
*/
/* 
 stot_1 = number of occupied patches in non-missing data for year t      
 npatch = number of patches
 x0[]     = state of patches at time 0
 x1[]     = state of patches at time 1
 x1p[]  = proposed states at time 1
 x2[]   = states at time 2
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 S1[]   = vector of connectivities of all patches from (current) states for time 2
 S1p[]   = vector of connectivities calculated from x1p, the vector of proposed states
 theta2[] = parameters to calculate probs for time 2
 logJxp = workarray of length npatch containing log(J(xp|x))
INPUT:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1)
 logtrprobx1_x2[] = log transition probabilities for time 2
   (for transition of x1 to x2 time 1 to 2)

OUTPUT
 x1p vector of proposed states
 for missing entries only (but the other probx0_x1 values remain unchanged anyway):
 logtrprobx0_x1p[] = log transition probabilities for x0 to x1p
				 (first calculated and used as log transition probabilities for x0 to 1-x1)
 logtrprobx1p_x2 =  log transition probabilities for x1p to x2

 
 */
{
  int i, k, t, stot, kbegin;
  double f, logr, probi, logJxi, logprobx0_x1pp, logprobx1pp_x2;
	state_t occupied =1;

	   for (i=1;i<=npatch;i++ ) x1p[i] = x1[i];


		 kbegin =   *ibeginmissing;
		 t = ti_missinglist[kbegin][1];

  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];
       logtrprobx0_x1p[i] = log(1-exp(logtrprobx0_x1[i])); /* probs for states x1p = 1-x1 */
		   logtrprobx1p_x2[i] = logptrans(x2[i], occupied-x1[i], S1[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
			 }

		 do{
		 stot = stot_1;
  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];

/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S1 is kept constant during the proposal generation
 			 f = prob("x1p"=1-x1)/prob(x1) =prob(1-x1)/prob(x1) =
			  ( (1-probx0_x1) * probx1p_x2) /	 (probx0_x1 * probx1_x2 )
  From this calculate P(x1p=1|x1) and logJxp
*/
		  f = exp( logtrprobx0_x1p[i] + logtrprobx1p_x2[i] -
			 			         (logtrprobx0_x1[i]  + logtrprobx1_x2[i]) );

		 if (x1[i]) {
 /* x1switched = "x1p" = 1-x1 = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1p" = 1-x1 = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
       x1p[i] = (probi > urandread(&seed)  );
			 if (x1p[i]) {
			   logJxp[i] = log(probi);
  			 stot++;
			 }
			 else {
			   logJxp[i] = log(1 - probi);
			 }
	   }
 	 } while (stot == 0);

   calc_S2( S1p, npatch, x1p, ADa);

	 logr = 0.0;
   for ( i = 1; i <= npatch; i++){
 	    logtrprobx1p_x2[i] = logptrans(x2[i], x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
      logr += logtrprobx1p_x2[i] - logtrprobx1_x2[i];
   }

   for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];

/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S1p is kept constant.
	This oddsratio is needed to calculate the jump 
 			 f = prob("x1pp"=1-x1p)/prob(x1p) =prob(1-x1p)/prob(x1p) =
			  ( (1-probx0_x1p) * probx1pp_x2) /	 (probx0_x1p * probx1p_x2 )
  From this calculate P(x1=1|x1p) and logJxi
*/
	 	  if (x1[i] == x1p[i]) {
  			 logprobx0_x1pp = logtrprobx0_x1p[i];
  			 logtrprobx0_x1p[i] = logtrprobx0_x1[i];
			}
			else {
  			 logprobx0_x1pp = logtrprobx0_x1[i];
			}
/*		logprobx0_x1pp = log(1-exp(logtrprobx0_x1p[i]));*/ /* probs for states x1pp = 1-x1p */
			logprobx1pp_x2 = logptrans(x2[i], occupied-x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
		  f = exp( logprobx0_x1pp  + logprobx1pp_x2 -
			 			         (logtrprobx0_x1p[i] + logtrprobx1p_x2[i]) );

		 if (x1p[i]) {
 /* x1switched = "x1pp" = 1-x1p = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1pp" = 1-x1p = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
			 if (x1[i]) {
			   logJxi = log(probi);
			 }
			 else {
			   logJxi = log(1 - probi);
			 }
	 		 if (x1[i]== x1p[i]){
			   logr += logJxi - logJxp[i];
			 }
			 else {
			   logr += logtrprobx0_x1p[i] - logJxp[i] - (logtrprobx0_x1[i] - logJxi);
			 }
	 }

	 *ibeginmissing = k;

   return exp(logr);
}

#endif /*QUICK == 2*/

void  Gibbs_one_patch(int npatch, state_t x1[], state_t x1p[], state_t x2[],
	 double logtrprobx0_x1[], double **logtrprobx1_x2,
	 double **logtrprobx1p_x2,
   double **S1, double **S1p, double logA[], double** logE1_E, double** ADa, \
   double theta2[], int npar, double** X, int *nxpar,
	 int *ibeginmissing, int **ti_missinglist)
/*  Does a Gibbs step for a single patch  i in year 1
Generates a proposal column (not for first or last column ) and
    returns the jump log_odds for proposed column of patches at time 1
       to the current column of patches at time 1
                               given states at time 0 and time 2 
     odds = prob(x1p)/prob(x1c)

*/
/* 
 stot_1 = number of occupied patches in non-missing data for year t      
 npatch = number of patches
 x0[]     = state of patches at time 0
 x1[]     = state of patches at time 1
 x1p[]  = proposed states at time 1
 x2[]   = states at time 2
 A[]     = area of patch i
 ADa   = matrix of (Area of patch j) * exp(-Distance of patch j to patch i/alpha)
 X[i][j]  =  extra explanatory variable j for patch i  , 
 S1[]   = vector of connectivities of all patches from (current) states for time 2
 S1p[]   = vector of connectivities calculated from x1p, the vector of proposed states
 theta2[] = parameters to calculate probs for time 2
 logtrprobx1p_x2 = work array only!!!
INPUT/output:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1)
 logtrprobx1_x2[] = log transition probabilities for time 2
   (for transition of x1 to x2 time 1 to 2)

 logtrprobx1p_x2[] = work array =	log transition probabilities for x1p to x2 at time 2
        (for transition of x1p to x2 for time 1 to 2, i.e. for new states x1p to x2) 
 S1 , S1p and logtrprobx1_x2 and logtrprobx1p_x2 are vectors but
 declared as double **  becouse they are swapped in the function.
 Call with &S1 ect.

 */
{
  int i, i1, k;
  double f, probi;
	double lprobx0_x1, lprobx0_x1p, lprobx1_x2, lprobx1p_x2;
	state_t occupied =1;

		 k =   *ibeginmissing;

  /*  t = ti_missinglist[k][1];   */
		 i1 = ti_missinglist[k][2];
/* calculate f = the  conditional oddsratio, 
 			 f = prob("x1p")/prob(x1) = prob(1-x1)/prob(x1) =
*/
		 lprobx0_x1 = logtrprobx0_x1[i1];
		 lprobx0_x1p = log(1-exp(logtrprobx0_x1[i1])); /* probs for states 1-x1 */
	   lprobx1_x2 = 0.0;
	   lprobx1p_x2 = 0.0;
	   for ( i = 1; i <= npatch; i++){
    		 lprobx1_x2  +=  (*logtrprobx1_x2)[i];

		     if (i !=i1 ){ 
		       if (x1[i1]) (*S1p)[i] = (*S1)[i] - ADa[i][i1]; else (*S1p)[i] = (*S1)[i] + ADa[i][i1];
  	 	     lprobx1p_x2 += ((*logtrprobx1p_x2)[i] = logptrans(x2[i], x1[i], (*S1p)[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar) );
				 }  
				 else {
				 /* i == i1*/ 
				 (*S1p)[i] = (*S1)[i];
	 	     lprobx1p_x2 += ((*logtrprobx1p_x2)[i] = logptrans(x2[i], occupied - x1[i], (*S1p)[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar) );
				 }
		 
     }


     f = exp(lprobx0_x1p + lprobx1p_x2 - (lprobx0_x1 + lprobx1_x2));


		 if (x1[i1]) {
 /* x1switched = "x1p" = 1-x1 = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1p" = 1-x1 = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
       x1p[i1] = (probi > urandread(&seed)  );
			 if (x1[i1] != x1p[i1]) {
			   x1[i1] = x1p[i1]; /* instead of a swap */
			   logtrprobx0_x1[i1] = lprobx0_x1p; /* = logtrprobx0_x1p[i1] == swap_pd(logprobx0_x1p,logprobx0_x1);*/
			   swap_pd(&(*logtrprobx1p_x2),&(*logtrprobx1_x2));
			   swap_pd(&(*S1p),&(*S1));
			 }
       	 *ibeginmissing = k+1;
}



void Gibbs_column_last_t(state_t x1[], state_t x1p[],
	 double logtrprobx0_x1[],	 
	 int *ibeginmissing, int **ti_missinglist, int stot_1)
/*  Generates a proposal column (not for first or last column ) and
    returns the jump log_odds for proposed column of patches at time 1
       to the current column of patches at time 1
                               given states at time 0 and time 2 
     odds = prob(x1p)/prob(x1)

*/
/* 
 stot_1 = number of occupied patches in non-missing data for year t      
 x1[]     = state of patches at time 1 (input)
 x1p[]  = proposed states at time 1   (work array only)
INPUT/OUTPUT for missing entries only:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1) 
*/
{
  int i, k, t, stot, kbegin;
  double lprob;
	# if CHECK
	double ptr;
	int xm;
	# endif


		 kbegin =   *ibeginmissing;
		 t = ti_missinglist[kbegin][1];

		 do{
			 stot = stot_1;
	  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
				 i = ti_missinglist[k][2];
				 if (x1[i]) {
				   lprob = logtrprobx0_x1[i];
				 }
				 else {
				   lprob = log(1-exp(logtrprobx0_x1[i]));
				 }
	       x1p[i] = (lprob > log(urandread(&seed))  );
				 # if CHECK

				 fscanf(par,  " %lf %d " ,&ptr, &xm);
				 fprintf(mon, " difflogp =  %20.17lf  p(1)= %20.17lf ptr = %20.17lf xs = %3d\n", 
				 lprob-log(ptr),  exp(lprob),ptr, x1p[i]);

				   if (fabs(lprob-log(ptr)) > 1.0E-5) {
					 fprintf(mon," LARGE DIFP\n");
					 }
	
				   if (xm!= x1p[i]){
					 fprintf(mon," DIFF_STATE last_t\n");
					 }
					 # endif


  			 if (x1p[i]) stot++;
	     }
 	   } while (stot == 0);


		 kbegin =   *ibeginmissing;
		 t = ti_missinglist[kbegin][1];

  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];
		   if (x1[i] !=x1p[i]) {
			   x1[i] = x1p[i];        /* instead of a swap */
			   logtrprobx0_x1[i] = log(1-exp(logtrprobx0_x1[i]));
			 } 
		 }
  	

 	 *ibeginmissing = k;
}


void calc_logtrprob(int npatch, int startyear, int endyear, state_t **Xs, \
	double **S, double *logA, double** logE1_E, double **theta, int npar, double **X, int *nxpar, double **logtrprob)
/* calculates the transition probabilities logtrprob from startyear to endyear for all patches

   state_t  **Xs;  states with front times and imputed missings 
   double *A,       vector of Areas  
	        **S,      times x patches matrix of connectivities   
          **X,      patches x nvars matrix of extra explanatory variables  
          **theta,  times x npar matrix of parameter values  
          **logtrprob,times x npatch matrix of transition probabilities
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram
startyear and endyear must therefore lie in the range -L , nyear

If DEBUG
output: 
logtrprob transition probabilities for t = startyear to endyear (all npatch patches)
			
*/
{ 
	int t,i;
	  for (t =startyear;t<=endyear ;t++ ) {
   		for (i=1;i<=npatch ;i++ )
				logtrprob[t][i] = logptrans(Xs[t][i], Xs[t-1][i], S[t-1][i], logA[i], logE1_E[i], theta[t], npar,\
					 X[i], nxpar);
	 }
}


double floglik0(int npatch, int startyear, int endyear, double **logtrprob)
/* loglik calculates the loglikelihood of the model 
		from years startyear to and including endyear
		using existing logs of transition probabilities in logtrprob

   double  **logtrprob,times x npatch matrix of transition probabilities
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram
startyear and endyear must therefore lie in the range -L , nyear

		
*/
{ 
	int t,i;
	double llik;

	llik = 0;

	  for (t =startyear;t<=endyear ;t++ ) 
   		for (i=1;i<=npatch ;i++ ) llik += logtrprob[t][i];			
			

	 return llik;
}

void init_Xs_S2(int nyear, int npatch, int **idata, int **ti_missinglist, int nmissing, \
 state_t **Xs,  int Xs_tot[], int L, int StartPattern, double **S, double **ADa,\
 double *logA, double** logE1_E, double **ptheta0, int npar, double **X, int *nxpar, double f_ini,
 int propose_t, double** logtrprob)
 /*initialize
   - Xs (the states) from data (idata) and imputes the missings by simple imputation
	 					using the initial model parameters  ptheta0
	 - S  (the connectivities) from Xs, A, D (distances) and alfa
	 Xs and S are (-L...nyear) by (1...npatch)  
*/	
{
	int t,i,k, ibeginmissing;
	state_t stot;
	double *trprob;

  trprob    = dvector(1,npatch);

  for (t =1;t<=nyear ;t++ )	{for (i=1;i<=npatch ;i++ )	Xs[t][i] =idata[t][i];}


	for (k=1; k<= nmissing;  k++ ) {
/* fill missings in all data years
   (first year stays, rest replaced by forward filling*/
    t = ti_missinglist[k][1];
		i = ti_missinglist[k][2];
		# if ZEROONE
		Xs[t][i]= (urand(&seed) < f_ini);
		#else
		Xs[t][i]= urand(&seed);
		# endif
	}



/* impute missings for years 2 to nyear by forward filling */
  /* find beginning of second or later year */ 
	  for (k=1; ti_missinglist[k][1] == 1;  k++ ) ;
 
		ibeginmissing  = k;
	  while ((t=ti_missinglist[ibeginmissing][1]) <= nyear) {

	  calc_S2( S[t-1], npatch , Xs[t-1], ADa);
    calc_logtrprob(npatch, t , t, Xs, S, logA, logE1_E, ptheta0, npar, X, nxpar, logtrprob);

 	  Gibbs_column_last_t(Xs[t],  Xs[propose_t],   
			    logtrprob[t], 
					&ibeginmissing, ti_missinglist, Xs_tot[t]);
			 
  	}

 


/* impute missings for preyears 0 to -L*/

	 if ((StartPattern == 111)||(StartPattern == -111) ) {
	 	
	 		 	/*  start with ones on t=-L */
				t = -L;
				for (i=1;i<=npatch ;i++ )  Xs[t][i] = 1;
				 
				/*  then forward filling */
			  for (t = -L+1;t<=0 ;t++ ) {
			    calc_S2( S[t-1], npatch , Xs[t-1], ADa);
			   	for (i=1;i<=npatch ;i++ ){
			        trprob[i] = exp(logptrans( 1, Xs[t-1][i], S[t-1][i], logA[i], logE1_E[i], ptheta0[0], npar, X[i], nxpar));
			        #if !ZEROONE
			           Xs[t][i] = trprob[i];
			        #endif
			    }
			   #if   ZEROONE
			    do {
			      stot = 0;
			   	  for (i=1;i<=npatch ;i++ ){
			        Xs[t][i] = (trprob[i]>urand(&seed));
			        stot +=  Xs[t][i];
			       }
			    } while ( stot < 4 );
			   #endif
			  }
        calc_S2( S[0], npatch , Xs[0], ADa);
  }  /* end if StartPattern */

	else {
		/*  backwards filling */
	  for (t =0;t>=-L ;t-- ) {
	    calc_S2( S[t+1], npatch , Xs[t+1], ADa);
	   	for (i=1;i<=npatch ;i++ ){
	        trprob[i] = exp(logptrans( 1, Xs[t+1][i], S[t+1][i], logA[i], logE1_E[i], ptheta0[0], npar, X[i], nxpar));
	        #if !ZEROONE
	           Xs[t][i] = trprob[t][i];
	        #endif
	    }
	   #if   ZEROONE
	    do {
	      stot = 0;
	   	  for (i=1;i<=npatch ;i++ ){
	        Xs[t][i] = (trprob[i]>urand(&seed));
	        stot +=  Xs[t][i];
	       }
	    } while (stot < 4 );
	   #endif
	  }
    calc_S2( S[-L], npatch , Xs[-L], ADa);
	}
	/* calculate connectivity S[t] for year t (t= 1 ... nyear) */

  for (t =1;t<=nyear ;t++ ) calc_S2( S[t], npatch , Xs[t], ADa);
  calc_logtrprob(npatch, -L+1, nyear, Xs, S, logA, logE1_E, ptheta0, npar, X, nxpar, logtrprob);
  
  fprintf(out," connect S (1...nyear) x npatch\n");
	for (i=1;i<=npatch ;i++ )fprintf(out, "%10d",i);
  fprintfdmatrix(out,S,1,nyear,1,npatch, "%10.6lf");

  free_dvector(trprob,1,npatch);
}




# endif /*QUICK == 2 || QUICK == 3*/

void read_Dist(FILE *dis, int npatch, double **D, double **B)
/*
format file dis
   1 2 1.0   4.0
	 1 3 0.5  10.0
	 0 0 0.0   0.0
	 
	 from_patch to_patch Bij Dij ending file with 0 0 0.0 0.0
	 Bij = barrier information (NB 1 = no barrier; 0 = complete barrier)
	 If an (i,j) pair of patches is not in the file Bij = 0 and Dij = HUGE,
	        i.e. a complete barrier between i and j is assumed

*/
{
	int i,j;
	double Dij, Bij;
	for (i=1; i<=npatch; i++ ) {
		for (j=1; j<i; j++ ) {
			D[i][j] = D[j][i] = HUGE;
			B[i][j] = B[j][i] = 0.0;
		}
		D[i][i] = B[i][i] = 0.0;
	}
	while (i>0) {
     fscanf(dis, "%d %d %lf %lf", &i, &j, &Bij, &Dij);
		 if (i>0) {
		 	 D[i][j] = D[j][i] = Dij;
		 	 B[i][j] = B[j][i] = Bij;
		 }
	}




}

void calc_ADa(double **ADa, int npatch, double *A, double **D, double **B, double alpha, double b, int With_area_effect)
/* Calculates the matrix ADa from the vector Area A , distance matrix D and alpha
   B is a barrier matrix (1.0 for no barrier between patch i and j; for complete barrier)
*/
{
	int i, j;
	if (With_area_effect) {
		for (i=1; i<=npatch ; i++)
		  for (j=1; j<=npatch; j++) ADa[i][j]=pow(A[j],b)*B[i][j]*exp(-D[i][j]/alpha);
	}
	else {
		for (i=1; i<=npatch ; i++)
		  for (j=1; j<=npatch; j++) ADa[i][j]=B[i][j]*exp(-D[i][j]/alpha);
	}
	for (i=1; i<=npatch ; i++) ADa[i][i]=0.0;

}

void calc_S2(double *S, int npatch, state_t *Xs, double **ADa)
/* calculates the connectivity S (an npatch vector) for all patches
 from an npatch-vector of states and matrix ADa with elements (i!=j)
     A[j]* exp(-D[i][j]/alpha);
       or (without area effect
     exp(-D[i][j]/alpha);
		 and
		 ADa[i][i]= 0
 
 Output S!
 */
{
  int i,j;
  double Si;
  for ( i=1;i<=npatch ;i++ ) {
  	Si = 0.0;
  	for ( j=1;j<=npatch ;j++ ) Si += Xs[j] * ADa[i][j];
  	S[i] = Si;	
  }
}

# if QUICK== 3

double propose_column0(int npatch,
   state_t x0[], state_t x1[], state_t x1p[], state_t x2[], 
	 double logtrprobx0_x1[], double logtrprobx1_x2[], 
	 double logtrprobx0_x1p[], double logtrprobx1p_x2[],
   double S0[], double S1p[], double** ADa, double logA[], double** logE1_E, \
   double theta2[], int npar, double** X, int *nxpar, double logJumpratio[])
/*  Generates a proposal column (not for first or last column ) and
    returns the jump log_odds for proposed column of patches at time 1
       to the current column of patches at time 1
                               given states at time 0 and time 2
    The proposal generation is independent of the data at time 1
		because S0 is used (instead of S1)															  
     odds = prob(x1p)/prob(x1)

*/
/*       
 npatch = number of patches
 x0[]     = state of patches at time 0
 x1[]     = state of patches at time 1
 x1p[]  = proposed states at time 1
 x2[]   = states at time 2
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 S0[]   = vector of connectivities of all patches from (current) states for time 2
 theta2[] = parameters to calculate probs for time 2
 logtrprobx1p_x2 = work array only!!!
 logJumpratio = workarray of length npatch (log(J(x1p)/J(x1))
INPUT:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1)
 logtrprobx1_x2[] = log transition probabilities for time 2 [NOT used!!!]
   (for transition of x1 to x2 time 1 to 2)

OUTPUT: logtrprobx0_x1p[] = log transition probabilities for x0 to x1p
				 (first calculated and used as log transition probabilities for x0 to 1-x1)
*/
{
  int i, stot;
  double f, logf, logr,  probi;
	state_t occupied =1;

	   for ( i = 1; i <= npatch; i++){
       logtrprobx0_x1p[i] = log(1-exp(logtrprobx0_x1[i])); /* probs for states 1-x1 */
		   logtrprobx1p_x2[i] = logptrans(x2[i], occupied-x0[i], S0[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
		 }

		 do{
		 stot = 0;
	   for ( i = 1; i <= npatch; i++){

/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S1 is kept constant during the proposal generation
 			 f = prob("x1p")/prob(x1) = prob(1-x1)/prob(x1) =
			  ( (1-probx0_x1) * probx1p_x2) /	 (probx0_x1 * probx1_x2 )
			 f = exp( logtrprobx0_x1p[i] + logtrprobx1p_x2[i] -
			 			         (logtrprobx0_x1[i]  + logtrprobx1_x2[i]) );
	*/
      logf =   logtrprobx0_x1p[i] - logtrprobx0_x1[i] ;

			if (x1[i] == x0[i]) {
					logf += logtrprobx1p_x2[i];
					if (x1[i] == x2[i]) {
					  logf -= logtrprobx0_x1[i];
					}
					else {
					  logf -= logtrprobx0_x1p[i];
					}
			}
			else {	
			  /*x1[i]!=x0[i]*/
					logf -= logtrprobx1p_x2[i];
					if (x1[i] == x2[i]) {
					  logf += logtrprobx0_x1[i];
					}
					else {
					  logf += logtrprobx0_x1p[i];
					}
			}

		  f = exp( logf );

	   if (x1[i]) {
 /* x1switched = "x1p" = 1-x1 = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1p" = 1-x1 = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
       x1p[i] = (probi > urandread(&seed)  );
			 if (x1p[i]) stot++;
			 if (x1[i] != x1p[i]) {
			   logJumpratio[i] = log(f);
			 }
			 else {
   		   logtrprobx0_x1p[i] = logtrprobx0_x1[i];
			 }
	   }
 	 } while (stot == 0);

/* End of jump prob. ratio Now work on density ratio*/

   calc_S2( S1p, npatch, x1p, ADa);
	 logr = 0.0;
   for ( i = 1; i <= npatch; i++){
 	    logtrprobx1p_x2[i] = logptrans(x2[i], x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
	    if (x1[i]!=x1p[i]){
			    logr += logtrprobx0_x1p[i] + logtrprobx1p_x2[i] -
            		(logtrprobx0_x1[i]+logtrprobx1_x2[i]) - logJumpratio[i];
			}
			else {
			    logr += logtrprobx1p_x2[i] - logtrprobx1_x2[i];
			}
   }




   return exp(logr);
}


double propose_column0_data(int npatch, state_t x0[], state_t x1[], state_t x1p[], state_t x2[],
	 double logtrprobx0_x1[],  double logtrprobx1_x2[],
	 double logtrprobx0_x1p[],  double logtrprobx1p_x2[],
   double S0[], double S1p[], double** ADa, double logA[], double** logE1_E, \
   double theta2[], int npar, double** X, int *nxpar,
	 int *ibeginmissing, int **ti_missinglist, int stot_1, double logJumpratio[])
/*  Generates a proposal column (not for first or last column ) and
    returns the jump log_odds for proposed column of patches at time 1
       to the current column of patches at time 1
                               given states at time 0 and time 2 
    The proposal generation is independent of the data at time 1
		because S0 is used (instead of S1)															  
      odds = prob(x1p)/prob(x1c)

*/
/* 
 stot_1 = number of occupied patches in non-missing data for year t      
 npatch = number of patches
 x0[]     = state of patches at time 0
 x1[]     = state of patches at time 1
 x1p[]  = proposed states at time 1
 x2[]   = states at time 2
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 S0[]   = vector of connectivities of all patches from (current) states for time 2
 theta2[] = parameters to calculate probs for time 2
 logtrprobx1p_x2 = work array only!!!
INPUT:
 logtrprobx0_x1[] = log transition probabilities for time 1
   (for transition of x0 to x1 for time 0 to 1)
 logtrprobx1_x2[] = log transition probabilities for time 2
   (for transition of x1 to x2 time 1 to 2)

OUTPUT
 x1p vector of proposed states
 for missing entries only (but the other probx0_x1 values remain unchanged anyway):
 logtrprobx0_x1p[] = log transition probabilities for x0 to x1p
				 (first calculated and used as log transition probabilities for x0 to 1-x1)
*/
{
  int i, k, t, stot, kbegin;
  double f, logf, logr, probi;
	state_t occupied =1;

	   for (i=1;i<=npatch;i++ ) x1p[i] = x1[i];


		 kbegin =   *ibeginmissing;
		 t = ti_missinglist[kbegin][1];

  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];
       logtrprobx0_x1p[i] = log(1-exp(logtrprobx0_x1[i])); /* probs for states 1-x1 */
		   logtrprobx1p_x2[i] = logptrans(x2[i], occupied-x0[i], S0[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
			 }

		 do{
		 stot = stot_1;
  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];

/* calculate f = the approximate conditional oddsratio, approximate because
  the connectivity S0 is kept constant during the proposal generation
 			 f = prob("x1p"=1-x1)/prob(x1) =prob(1-x1)/prob(x1) =
			  ( (1-probx0_x1) * probx1p_x2) /	 (probx0_x1 * probx1_x2 )

		  f = exp( logtrprobx0_x1p[i] + logtrprobx1p_x2[i] -
			 			         (logtrprobx0_x1[i]  + logtrprobx1_x2[i]) );

*/

      logf =   logtrprobx0_x1p[i] - logtrprobx0_x1[i] ;

			if (x1[i] == x0[i]) {
					logf += logtrprobx1p_x2[i];
					if (x1[i] == x2[i]) {
					  logf -= logtrprobx0_x1[i];
					}
					else {
					  logf -= logtrprobx0_x1p[i];
					}
			}
			else {	
			  /*x1[i]!=x0[i]*/
					logf -= logtrprobx1p_x2[i];
					if (x1[i] == x2[i]) {
					  logf += logtrprobx0_x1[i];
					}
					else {
					  logf += logtrprobx0_x1p[i];
					}
			}

		  f = exp( logf );



		 if (x1[i]) {
 /* x1switched = "x1p" = 1-x1 = 0 thus: f = p(0)/p(1) and p(1) = 1/(1+f)*/
					probi = 1/(1+f);
			 }
			 else {
 /* x1switched = "x1p" = 1-x1 = 1 thus: f = p(1)/p(0) and p(1) = f/(1+f)*/
					probi = f/(1+f);
			 }
       x1p[i] = (probi > urandread(&seed)  );
			 if (x1p[i]) stot++;
			 if (x1[i] != x1p[i]) {
			   logJumpratio[i] = log(f);
			 }
			 else {
			   logtrprobx0_x1p[i] = logtrprobx0_x1[i];
			 }
	   }
 	 } while (stot == 0);

/* End of jump prob. ratio Now work on density ratio*/
   calc_S2( S1p, npatch, x1p, ADa);

	 logr = 0.0;

   for ( i = 1; i <= npatch; i++){
 	  logtrprobx1p_x2[i] = logptrans(x2[i], x1p[i], S1p[i], logA[i], logE1_E[i], theta2, npar, X[i], nxpar);
	  logr += logtrprobx1p_x2[i] - logtrprobx1_x2[i];
   }


  	 for ( k = kbegin; ti_missinglist[k][1] == t; k++){
			 i = ti_missinglist[k][2];
			 if (x1[i] != x1p[i]) {
          logr += logtrprobx0_x1p[i] - logtrprobx0_x1[i] - logJumpratio[i];
			 }
		 }


	 *ibeginmissing = k;
   return exp(logr);
}

# endif /*QUICK == 3*/

