
/*
  fmetapop.c: main program for baysian fitting the patch occupancy model of
   metapopulation dynamics
   C.J.F. ter Braak, CBW, Wageningen
   c.j.f.terbraak@plant.wag-ur.nl
   June 27, 2000
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <float.h>
#include <limits.h>

/* numerical recipes: */



#include <nrutil.h>
#include <nr.h>

#include "fmetapop.h"
#include "MC_patches.h"

FILE *out, *par, *dat, *mon, *dis;
long seed;
int Stochastic_EM=0, Use_J_model=0;
		/* numerical cut-off of log probabilities in logptrans and MC_patches, 
		set by EPS in fmetapop.h  */
double logEPS, log1_EPS;

main(int argc, char *argv[])
{

/* parameters */
int
  npatch,          /* number of patches */
  nyear,           /* number of years */
  L ,             /* the number of pretimes is L+1 */
	L3,             /* L+3*/
	propose_t,     /* index for row in which the proposal resides*/
	propose_t1,     /* index for row in which the proposal trprob for t+1 resides*/
	StartPattern, /* indicator which spatial configuration is used at time -L; if negative the pattern is not updated */
	With_area_effect, /* indicator whether area enters the formula for the connectivity  */
	nhpar,          /* number of hyper parameters */
  npar,            /*number of parameters in theta*/
  nmpar,            /*number of parameters in basic metapopulation model (usually 5)*/
	nxpar[2],         /* nxpar[0] = number of explanatory variables of E model, 
	                     nxpar[1] =  number of explanatory variables of C model, */
  nxparEC ; /* number of extra explanatory variables = nxpar[0] + nxpar[1] */

/* optimization characteristics */
long niter, nsubiterXs;
double 	T0=1,              /* simulated annealing constants*/
 				delta0= 1,
  			mu = 1000;
int 		takepri =1;				  /* whether or not prior is taken into account in acceptance ratio*/


   int **idata,   /*  input states read from file dat */
	     *Xs_tot,   /*  nyear - vector of number of occupied patches in data */
	 		 nmissing, /* number of missing values (-1) in idata*/
       **ti_missinglist, /* list of missing values (-1) in idata: col1,col2 =(time,patch)
														list ends with (INT_MAX,INT_MAX) */
		   *to_be_estimated; /* index of parameters in theta to be estimated from the data
			 											A parameter is estimated when, in the par file,
														the second value after the parameter (theta_ini_se) is nonzero.  
			 											The remaining parameters remain fixed (not estimated) */
   state_t  **Xs;   /*  states with front times and imputed missings */
   double *xcoor,         /* x geographic coordinates  */
          *ycoor,         /* y geographic coordinates   */
	        *A,         /* vector of Areas  */
					*logA,      /* vecotr of log Areas */
	        **Dist,      /* patches x patches matrix of distances   */
	        **Barriers,      /* patches x patches matrix of barriers Bij   */
					**ADa0,    	  /* patches x patches matrix of A*exp(-distances/alpha0)   */
					**ADa1,    	  /* patches x patches matrix of A*exp(-distances/alpha1)   */
	        **S0,         /* times x patches matrix of connectivities  for theta0 */
	        **S1,         /* times x patches matrix of connectivities, for proposal theta1   */
          **X,        /* patches x nvars matrix of extra explanatory variables  */
          **theta0,    /* times x npar matrix of parameter values  */
          **theta1,    /* times x npar matrix of proposal parameter values  */
          **trans_theta0,    /* times x npar matrix of transformed (e.g. log) parameter values  */
          **trans_theta1,    /* times x npar matrix of transformed proposal parameter values  */
					**ptheta0,   /* pointers to times x npar matrix of  parameter values  theta*/
					**ptheta1,   /* pointers to times x npar matrix of proposal parameter values  theta*/
					**trans_ptheta0,   /* pointers to times x npar matrix of  parameter values  theta*/
					**trans_ptheta1,   /* pointers to times x npar matrix of proposal parameter values  theta*/
					*hyper_theta, /* nhpar vector of hyper of parameters*/
					*theta_ini, /* npar vector of ini values of parameters*/
					*theta_opt, /* npar vector of estimated values of parametersin EM*/
					*theta_ini_se, /* npar vector of ini values of se's of ***transformed*** parameter values
														i.e. the coefficient of variation for log-transformed values */
					*theta_lower,  /* npar vector of lower bounds for theta  */
					*theta_upper,    /* npar vector of upper bounds for theta  */
					*trans_par,     /* parameters used in ftrans_theta and fbacktrans_theta,
					 									normally the mean of the predictor on the transformed scale   */
 					**trans_theta_prior,  /* npar x 2-3 matrix of parameters of the prior for trans_theta
															typically the first column is the mean and the second is the sd   */
          **trprob, /*  times x npatch matrix of transition probabilities*/
          **logtrprob0, /*  times x npatch matrix of log transition probabilities based on theta0 and S0 */
          **logtrprob1, /*  times x npatch matrix of log transition probabilities based on theta1 and S1 */
   				**Xs_mean,   /*  mean of states with front times and imputed missings */
					**cov_ini,  /* initial covariance matrix  */
										 /* Note that the initial covariance matrix must be for the 
												 transformed parameters */
					**cov0,  /* current covariance matrix  */
					**cov1,  /* proposed covariance matrix  */
					**chol_factor, /* choleski factor of current covariance matrix */
					*ranvector, /* random vector for generating proposals (workspace) */
					*theta_mean, /* posterior mean of theta estimate */
					*theta_se, /* posterior se of theta estimate */
					**theta_cov, /* posterior covariance of theta estimate */
					**logE1_E, /*work array in MC_patches*/
					**pab, /* proposal props for MH of sttes*/
					**dmat_pointer, /* used for swapping dmatrices */
					fdum;

	 int	 	indexalpha,     /* index in theta of alpha, the parameter that defines the connectivity S  */
					indexb,     /* index in theta of b, the power of A in the definition of the connectivity S  */
	    		estimate_alpha,  /* 1 if alpha has to be estimated */
					icov, /* indicator whether an initial covariance matrix is in the par file */
					read_Dij, /* indicator whether a distance file need to be read */
					nestimate;		 /* number of parameters to be estimated (length of ivector to_be_estimated  */


   int  i, j, k, t, lag, stot;
	 int indexnyear;  /* position in timissinglist of first missing patch in year nyear */
	 int idum =0, acc = 0;
   long ldum=0,
	 			ini_burn,   /* initial burn in for updating missings for initial parameter values */
				nsubitertheta, /* number of  iterations for theta given states Xs*/
				iter_theta,
	      iter;

	 double	logr, logE,	logC, lp,
					meanlogA,
					meanlogS,
	        f_ini,					 /* freq of occupancy in the data */
	        accept_p = -1., /* acceptance prob for MH of patches per year*/
	 				Eloglik0, /* expected loglikelihood of current parameter values */
					Eloglik1=-FLT_MAX,  /* expected loglikelihood  of proposal parameter values   */
					E2loglik0 =0,  /* variance of expected loglikelihood of current parameter values */
					E2loglik1 = 0,	 /* variance of expected loglikelihood of current parameter values */
	 				Eloglikmax=-FLT_MAX, /* maximum expected loglikelihood at theta_opt */
          scale, /* scale of covariance matrix to generate proposals */ 
	 				logprior0, /*  logprior of current parameter values */
	 				logprior1, /*  logprior of proposal*/
	 				loglik0,	 /* loglikelihood of current parameter values */
	 				loglik1; 	 /*  proposal loglikelihood*/


	 int startyear, endyear;


   /*  file opening */

   char surname[100], filename[100];
   double T, delta;

	 
   
   if (argc != 2) sprintf(surname,"fmetapop"  );
   else  strcpy(surname, *(argv+1));


   strcpy(filename, surname); strcat(filename, ".par");
   if ( ( par = fopen(filename, "r") ) == NULL )
         error(strcat("cannot open ", filename));

   strcpy(filename, surname); strcat(filename, ".dat");
   if ( ( dat = fopen(filename, "r") ) == NULL )
         error(strcat("cannot open ", filename));


   strcpy(filename, surname); strcat(filename, ".out");
   if ( ( out = fopen(filename, "w") ) == NULL )
         error(strcat("cannot open ", filename));

   strcpy(filename, surname); strcat(filename, ".mon");
   if ( ( mon = fopen(filename, "w") ) == NULL )
         error(strcat("cannot open ", filename));

/* begin of initialization */

	 logo_fmetapop(stderr);
	 logo_fmetapop(out);
		/* numerical cut-off of log probabilities in logptrans and MC_patches */
	 	logEPS = log(EPS);
  	log1_EPS =log(1-EPS);



  /* reading input data size   */
   readi(dat, out,  "npatch", 1, &npatch, 0, IHUGE, &idum, 0, 0);
   readi(dat, out,  "nyear", 1, &nyear, 0, IHUGE, &idum, 0, 0);

	/*initialize parameter values by reading from file par*/

   readi(par, out,  "nhpar", 1, &nhpar, 0, IHUGE, &idum, 0, 0);
   readi(par, out,  "nmpar", 1, &nmpar, 1, IHUGE, &idum, 0, 0);
   readi(par, out,  "nxparE", 1, &nxpar[0], 0, IHUGE, &idum, 0, 0);
   readi(par, out,  "nxparC", 1, &nxpar[1], 0, IHUGE, &idum, 0, 0);
   readi(par, out,  "With_area_effect", 1, &With_area_effect, 0, 1, &idum, 0, 0);

	 if (QUICK==0 && With_area_effect!=WITH_AREA_EFFECT){
   	   meta_message(stderr, " ** Warning **  Because QUICK == 0, the With_area_effect parameter is overridden by the compiled WITH_AREA_EFFECT parameter in the header file") ;
   	   meta_message(out, " ** Warning **  Because QUICK == 0, the With_area_effect parameter is overridden by the compiled WITH_AREA_EFFECT parameter in the header file") ;
   }
	 nxparEC = nxpar[0]+nxpar[1];
	 npar = nmpar + nxparEC;

	 theta_ini = dvector(1,npar);
	 theta_opt = dvector(1,npar);
	 theta_ini_se = dvector(1,npar);
	 theta_lower = dvector(1,npar);
	 theta_upper = dvector(1,npar);
	 trans_par = dvector(1,npar);
	 trans_theta_prior = dmatrix(1,npar,1,3);

	 /*
	  Note on the second value se = (theta_ini_se)
		if (se == 0) then the parameter is not estimated
		if (se > 0 && icov ==0 [no covariance matrix specified) {
			se = the standard error in the assumed target distribution on 
					the transformed scale of theta
		}	
   */
   readf(par, out,  "e", 2, &(theta_ini[1]), 0.0, HUGE, &(theta_ini_se[1]), 0.0, HUGE);
   readf(par, out,  "x", 2, &(theta_ini[2]), 0.0, HUGE, &(theta_ini_se[2]), 0.0, HUGE);
   readf(par, out,  "y", 2, &(theta_ini[3]), 0.0, HUGE, &(theta_ini_se[3]), 0.0, HUGE);
   readf(par, out,  "z", 2, &(theta_ini[4]), 0.0, HUGE, &(theta_ini_se[4]), 0.0, HUGE);
   readf(par, out,  "alpha", 2, &(theta_ini[5]), 0.0, HUGE, &(theta_ini_se[5]), 0.0, HUGE);
   readf(par, out,  "b", 2, &(theta_ini[6]), 0.0, HUGE, &(theta_ini_se[6]), 0.0, HUGE);
  

   indexalpha =5;
   indexb = 6;
 /*	 alpha = theta_ini[indexalpha]; b = theta_ini[indexb];*/
 		if (theta_ini_se[indexalpha]> FLT_MIN || theta_ini_se[indexb]> FLT_MIN) estimate_alpha = 1; else estimate_alpha = 0;
	 for (k=nmpar+1;k<=npar ;k++ ) {
     readf(par, out,  "x[k]", 2, &(theta_ini[k]), -HUGE, HUGE, &(theta_ini_se[k]), 0.0, HUGE);
	 }

	 readi(par, out,  "read_Dij", 1, &read_Dij, 0, 2, &idum, 0, 0);

	 if (read_Dij) {
		 strcpy(filename, surname); strcat(filename, ".dis");
	   if ( ( dis = fopen(filename, "r") ) == NULL )
	         error(strcat("cannot open ", filename));
	 }



	 to_be_estimated = create_vindex(npar, theta_ini_se, &nestimate, 0.0); 
	 if (!nestimate) { meta_message(out, " No parameters to be estimated (theta_ini_se all zero) ");
	 }

	 /* initialize and set transpar */
	 for (k=1;k<=npar ;k++ ) trans_par[k]=0.0;
 
	 /* read trans_par (normally the mean on the transformed schale of predictor
	  end with 0 0*/
/* disable trans_par option	 
 	 { double ft, fmean;
		 do {
	   readf(par, out,  "trans_par", 2, &ft, -HUGE, HUGE, &fmean, -HUGE, HUGE);
		 k =(int)ft;
		 if (k>0) trans_par[k] = fmean;
		 } while (k>0);
	 }
*/

   readi(par, out,  "icov", 1, &icov, 0, 1, &idum, 0, 0);

	 cov_ini = dmatrix(1, nestimate, 1, nestimate);
	 cov0   = dmatrix(1, nestimate, 1, nestimate);
	 cov1   = dmatrix(1, nestimate, 1, nestimate);
	 chol_factor   = dmatrix(1, nestimate, 1, nestimate);
	 ranvector = dvector(1,nestimate);
	 theta_mean = dvector(1,nestimate);
	 theta_se = dvector(1,nestimate);
	 theta_cov = dmatrix(1, nestimate, 1, nestimate);

	 /* Note that the covariance matrix of the assumed target distribution is for the 
	 transformed parameters */

	 if (icov){
	 	/* choleski of cov_ini */
		 read_lower_matrix(par, cov_ini, nestimate);
		 cholesky(cov_ini, chol_factor, nestimate);
	 }
	 else {
	 	 for (k=1;k<=nestimate ;k++ ) theta_se[k] = theta_ini_se[to_be_estimated[k]];
	   set_cov_diag(chol_factor, nestimate, theta_se);
		 /* set cov_ini to diag(theta_se^2) */
		 multiply_dmatrices(chol_factor, chol_factor, cov_ini, nestimate, nestimate,nestimate);
	 } 	 

     readf(par, out,  "scale", 1, &scale, 0.0, HUGE, &fdum, 0.0, 0.0);

/*
  fprintfdmatrix(out, cov_ini,1,nestimate,1,nestimate, "%10.3lf");
   fprintfdmatrix(out, chol_factor,1,nestimate,1,nestimate, "%11.3lf");
	 transpose_dmatrix(chol_factor, cov1, nestimate,nestimate);
   fprintfdmatrix(out, cov1,1,nestimate,1,nestimate, "%12.3lf");
	 multiply_dmatrices(chol_factor, cov1, cov0, nestimate, nestimate,nestimate);
   fprintfdmatrix(out, cov0,1,nestimate,1,nestimate, "%13.3lf");
*/	 
 /* Should we use Hanski 's approximate incidence model for year 1? */
   readi(par, out,  "Use_J_model", 1, &Use_J_model, 0, 1, &StartPattern, -IHUGE, IHUGE);
	 /* in the par-file L is the number of preyears; in the program the number of preyears is L+1; */
   readi(par, out,  "L", 2, &L, -1, IHUGE, &StartPattern, -IHUGE, IHUGE);
   readi(par, out,  "startyear", 2, &startyear, -L, nyear, &endyear, -L, nyear);
	 if (L==0) {
	 	StartPattern=-1;
	 }
	 else {
  	 L= L-1; /* The program uses -L to nyear; including year 0 */
	 } 
	
	 if (startyear == 0 && endyear == 0 ) {
	 	/* use all information : space and time */
		if (L==-1 && Use_J_model == 0) {
			 error("Cannot use spatial information with L == 0; change startyear or Use_J_model");
			 /* L==0: one cannot use spatial info*/
		}	 	else startyear = -L+1;
		endyear = nyear;
	 }
	 if (startyear == 1 && endyear == 0 ) {
	 	/* use turnover information only */
		if (nyear>1)  startyear = 2;
		else{
			 error("Cannot use turnover information only with nyear == 1; change startyear");
		}
				endyear = nyear;
	 }
 
	 if (Use_J_model) {
		 if (startyear !=1) {
      message(" startyear set to 1, because Use_J_model == 1");
		  startyear = 1;
		 }
		 if (endyear == 0) endyear = nyear;
	 }
   if (startyear <= -L){
	   message(" startyear too small; startyear set to smallest possible value (-L+2)");/*note that for the user L is number of preyears*/
		 startyear = -L+1;
	 } 


	 if (scale < FLT_MIN) scale = 2.4;		 /* the default given by Gelman et al 1995 p 334 
	 																				which is on choleskibasis (standard error basis)*/
	 /*adapt scale for preyear number: 
	  the MCMC believes there are nyear-startyear+1 years of data  whereas there are only nyear of data
		the implied variances of the parameters in MCMC are there for a factor nyear/nyear-startyear+1 smaller
		than the initial cov-matrix based on just nyear-1 of (turnover)data*/
	 if (startyear<0) scale  *= sqrt((nyear)/(nyear-startyear+1.0)) ;
	 multiply_dmat_const(chol_factor, scale/sqrt(nestimate),chol_factor, nestimate, nestimate) ;



	 readl(par, out,  "seed", 1, &seed, 1000, LONG_MAX, &ldum, 0, 0);
	 seed = -seed;
	 readl(par, out,  "niter", 2, &niter, 1, LONG_MAX, &ini_burn, 1, LONG_MAX); /* minimum nsubiter = 1*/
	 readl(par, out,  "nsubiter", 2, &nsubitertheta, 0, LONG_MAX, &nsubiterXs, 1, LONG_MAX);
	 readi(par, out,  "lag", 1, &lag, 1, IHUGE, &idum, 0, 0);
	 /* EM = 0 does not EM but MCMC  Other values do not work from version 0.5!!!!
	       = 1 does stochastic EM 
				 = -1 calculates the Expected loglik only 
	  takepri = 1 uses the prior in EM
							0 does not use the prior
		  
   simulated annealing parameters in Stochastic EM	 Do not work from version 0.5
	 T = T0 * exp (-(mu*i)/N)  
	 delta = delta0 *exp (-(mu*i)/N)   [shrinkage of proposal variability]	*/
/*
	 readi(par, out,  "Stochastic_EM", 2, &Stochastic_EM , -1, 1, &takepri, 0, 1);
   readf(par, out,  "mu", 1, &mu, 10 , HUGE, &fdum, 0.0, 0.0);
	 mu = log(mu);
   readf(par, out,  "T&delta", 2, &T0, 0 , 1, &delta0, 0, 1);
*/

   readf(par, out,  "bounds_e", 2, &theta_lower[1], -HUGE , HUGE, &theta_upper[1], -HUGE, HUGE);
   readf(par, out,  "bounds_x", 2, &theta_lower[2], -HUGE , HUGE, &theta_upper[2], -HUGE, HUGE);
   readf(par, out,  "bounds_y", 2, &theta_lower[3], -HUGE , HUGE, &theta_upper[3], -HUGE, HUGE);
   readf(par, out,  "bounds_z", 2, &theta_lower[4], -HUGE , HUGE, &theta_upper[4], -HUGE, HUGE);
   readf(par, out,  "bounds_alpha", 2, &theta_lower[5], -HUGE , HUGE, &theta_upper[5], -HUGE, HUGE);
   readf(par, out,  "bounds_b", 2, &theta_lower[6], -HUGE , HUGE, &theta_upper[6], -HUGE, HUGE);
	 for (k=nmpar+1;k<=npar ;k++ ) {
     readf(par, out,  "bounds_x[k]", 2, &(theta_lower[k]), -HUGE, HUGE, &(theta_upper[k]),-HUGE, HUGE);
	 }

	 /* transform lower and upperbounds to the transformed scale */
   #if QUICK >=2
	   /* theta is on log/logit scale*/
	   theta_lower[1] = log(theta_lower[1]);
	   theta_upper[1] = log(theta_upper[1]);
	   fdum = -log(theta_lower[3]);
	   theta_lower[3] = -log(theta_upper[3]);
   	 theta_upper[3] = fdum;
		 ftrans_theta2(theta_lower, theta_lower, npar, trans_par);
	   ftrans_theta2(theta_upper, theta_upper, npar, trans_par);
	  
	 #else  
	   /* theta is on original E/C scale*/
   	 ftrans_theta(theta_lower, theta_lower, npar, trans_par);
  	 ftrans_theta(theta_upper, theta_upper, npar, trans_par);

	 # endif


   readf(par, out,  "prior_trans_e", 2,      &trans_theta_prior[1][1],   -HUGE , HUGE, &trans_theta_prior[1][2], -HUGE, HUGE);
   readf(par, out,  "prior_trans_x", 2,      &trans_theta_prior[2][1],   -HUGE , HUGE, &trans_theta_prior[2][2], -HUGE, HUGE);
   readf(par, out,  "prior_trans_y", 2,     &trans_theta_prior[3][1],   -HUGE , HUGE, &trans_theta_prior[3][2], -HUGE, HUGE);
   readf(par, out,  "prior_trans_z", 2,      &trans_theta_prior[4][1],   -HUGE , HUGE, &trans_theta_prior[4][2], -HUGE, HUGE);
   readf(par, out,  "prior_trans_alpha", 2,  &trans_theta_prior[5][1],   -HUGE , HUGE, &trans_theta_prior[5][2], -HUGE, HUGE);
   readf(par, out,  "prior_trans_b", 2,  &trans_theta_prior[6][1],   -HUGE , HUGE, &trans_theta_prior[6][2], -HUGE, HUGE);
	 for (k=nmpar+1;k<=npar ;k++ ) {
     readf(par, out,  "prior_trans_x[k]", 2, &(trans_theta_prior[k][1]), -HUGE, HUGE, &(trans_theta_prior[k][2]),-HUGE, HUGE);
	 }

	
   # if !CHECK
	 fclose(par);
	 # endif

	 

   /* creating data vectors and matrices   */
	 L3 = L+3; /*extra storage for the proposal states and probabilities
	            AND an extra row that is not swapped around!!! */
	 propose_t = -(L+1);
	 propose_t1 = -(L+2);
   idata = imatrix(1,nyear,1,npatch);
   Xs_tot = ivector(1,nyear);
   Xs    = state_matrix(-L3,nyear,1,npatch);
   xcoor = dvector(1,npatch);
   ycoor = dvector(1,npatch);
   A     = dvector(1,npatch);
   logA     = dvector(1,npatch);
   Dist     = dmatrix(1,npatch,1,npatch);
   Barriers     = dmatrix(1,npatch,1,npatch);
   ADa0      = dmatrix(1,npatch,1,npatch);
   ADa1      = dmatrix(1,npatch,1,npatch);
   S0     = dmatrix(-L3,nyear,1,npatch);
   S1     = dmatrix(-L3,nyear,1,npatch);
   X     = dmatrix(1,npatch, 0,nxparEC);
   hyper_theta = dvector(0,nhpar);				 /* allow nhpar = 0 */
   theta0 = dmatrix(-L,nyear,1,npar);
   theta1 = dmatrix(-L,nyear,1,npar);
   trans_theta0 = dmatrix(-L,nyear,1,npar);
   trans_theta1 = dmatrix(-L,nyear,1,npar);
   ptheta0 = pdmatrix(-L,nyear,1,npar);
   ptheta1 = pdmatrix(-L,nyear,1,npar);
   trans_ptheta0 = pdmatrix(-L,nyear,1,npar);
   trans_ptheta1 = pdmatrix(-L,nyear,1,npar);
   trprob    = dmatrix(-L3,nyear,1,npatch);
   logtrprob0    = dmatrix(-L3,nyear,1,npatch);
   logtrprob1    = dmatrix(-L3,nyear,1,npatch);
	 logE1_E = dmatrix(1,npatch,0,1);
	 pab = dmatrix(1,npatch,0,3);


   /* reading input data   */

   if (!read_file(npatch, nyear, xcoor, ycoor, A, idata, nxparEC, X) ) exit(1);
   # if !CHECK
   	 fclose(dat);
	 # endif
	 meanlogA=0.0;
   for (i=1; i<=npatch;i++) meanlogA += (logA[i] = log(A[i]));
	 meanlogA /= npatch;

	 	fprintf(out, "\n\nCopy of dat-file \n x and y coordinates, Area and, if any, extra predictor variables\n") ;
	 for (i=1;i<=npatch ; i++) {
	 	fprintf(out, "\n %lf %lf %lf ", xcoor[i], ycoor[i],A[i]) ;
       for (j=1; j<=nxparEC ; j++ ) fprintf(out, "%lf ", X[i][j]);
	 }																 

   /*  create year-totals and list of missings in the data (idata) */

	 stot = sum_row_imatrix(nyear, npatch, idata, MISSING, Xs_tot);
 	 ti_missinglist = create_missinglist(nyear, npatch, idata, &nmissing);
	 indexnyear = nmissing+1;
	 for (j=nmissing;ti_missinglist[j][1]==nyear ;j-- ) indexnyear = j;
	 

	fprintf(out, "\n\n List of missings in occupancy data \n year  patch\n") ;
   fprintfimatrix(out, ti_missinglist,1,nmissing,1,2, "%5d");
	 f_ini = ((double)stot)/(nyear*npatch - nmissing);

	 if (Stochastic_EM!=0 && QUICK==0 && nmissing > 0 && startyear < 2 ) error( " Stochastic EM not feasible with an executable made with QUICK == 0 in header file  ");
	 if (Stochastic_EM!=1){
	  takepri=1; T0=1; delta0=1;
	 } 
	 if (Stochastic_EM==1 && nsubiterXs < 5 && nmissing > 0 && startyear < 2 ) nsubiterXs = 5;
	
	 if (Use_J_model && ti_missinglist[1][1]!=1) {
	 	/* no missings in first year */
    if (L>0)message(" For efficiency, L set to 0, because Use_J_model == 1 and no missings in first year");
	 	L = 0;
		StartPattern = -1;
	 }
 

 

   /* transform theta to the ones used for the proposal generation (trans_theta) and
	 		in the prior, and
	    copy theta_ini to theta0 and theta1  and idem for trans_theta*/

   #if QUICK >=2
	   /* theta is on log/logit scale*/
	   theta_ini[1] = log(theta_ini[1]);
	   theta_ini[3] = -log(theta_ini[3]);
  	 ftrans_theta2(theta_ini, trans_theta0[0], npar, trans_par);
	 #else  
	   /* theta is on original E/C scale*/
  	 ftrans_theta(theta_ini, trans_theta0[0], npar, trans_par);
	 # endif

   copy_theta_ini(nyear, L, npar, theta_ini, theta0, ptheta0);
   copy_theta_ini(nyear, L, npar, theta_ini, theta1, ptheta1);
   copy_theta_ini(nyear, L, npar, trans_theta0[0], trans_theta0, trans_ptheta0);
   copy_theta_ini(nyear, L, npar, trans_theta0[0], trans_theta1, trans_ptheta1);

	 /* use hereafter ptheta0 and trans_ptheta0, and ptheta1 and trans_ptheta1, 

	  ** Warning** : usage of theta0/1 lead to unfilled data 
		in particular: if !REGIONAL_STOCHAST then theta1[t][k] = unset	for t!=0
		
		*/


	 /* calculate distance matrix and ADa0 for theta_ini */
	 	 for (i=1; i<=npatch; i++){for (j=1; j<=npatch; j++) Barriers[i][j] = 1.0;}
	 	 for (i=1; i<=npatch; i++) Barriers[i][i] = 0.0;
	   calc_D(npatch, xcoor, ycoor, Dist);
	 
    if (read_Dij == 1) {
	 	 read_Dist(dis, npatch, Dist, Barriers);
	 	 }
	  else if (read_Dij == 2) { /*
		 Vos: combine read_Dij info in fmetapop.dis with xcoor, ycoor distances
	           if Dist == 0 in fmetapop.dis then use d(x-y) with Bij = 0.5 */
	 	  read_Dist(dis, npatch, ADa0, Barriers); /*ADa0 is work array here*/
	 	  for (i=1; i<=npatch; i++) {
		    for (j=1; j<i; j++) {
		      if (Barriers[i][j] < EPS && ADa0[i][j] >= HUGE ) { /* pair i j not in fmetapop.dis*/
	       		Barriers[i][j] = Barriers[j][i] = 0.5;
				  }
				  else {/* pair i,j in fmetapop.dis:*/
     			  Dist[i][j] = Dist[j][i] = ADa0[i][j];
				  }
		    }
		  }
	  }	 

	 calc_ADa(ADa0, npatch, A, Dist, Barriers, theta_ini[indexalpha], theta_ini[indexb], With_area_effect);
	  
/* -initialize Xs (the states) from data and simulation  
	   -calculate connectivity S[t] for year t (t= -L ... nyear) */

	 #if QUICK >= 2

		 #if !REGIONAL_STOCHAST
		  for (i=1;i<=npatch ;i++ ) {
				lp = 0.0;
				for (j= 1;j<= nxpar[0] ;j++ )  lp +=  ptheta0[0][nmpar+j]*X[i][j];
			  logE = DMIN(ptheta0[0][1] - ptheta0[0][2] * logA[i] + lp, log1_EPS); 
		    logE1_E[i][0] = logE = DMAX(logE, logEPS);
				logE1_E[i][1]= log(1.0 - exp(logE)); /* log(1-E) */

			}
		 #endif
     init_Xs_S2(nyear, npatch, idata, ti_missinglist, nmissing, Xs, Xs_tot, L, StartPattern, S0, ADa0,\
						 logA, logE1_E, ptheta0, npar, X, nxpar, f_ini, propose_t, logtrprob0);
		 
     loglik0 = floglik0(npatch,startyear, endyear, logtrprob0);

	 #elif QUICK == 1
	    init_Xs_S1(nyear, npatch, idata, ti_missinglist, nmissing, Xs, L, StartPattern, S0, ADa0,\
						 A, theta_ini, npar, X, nxpar, f_ini);
    	loglik0 	 = floglik(npatch, startyear, endyear, Xs, S0, A, ptheta0, npar, X, nxpar);
	 #else
      init_Xs_S(nyear, npatch, idata, ti_missinglist, nmissing, Xs, L, StartPattern, S0, Dist, indexalpha,\
						 A, theta_ini, npar, X, nxpar, f_ini);
    	loglik0 	 = floglik(npatch, startyear, endyear, Xs, S0, A, ptheta0, npar, X, nxpar);
	 #endif
	 /* copy S0 and ADa0 to S1 and ADa1 ; needed for case S remains constant */
   for (t=-L; t<=nyear;t++ ) {
	 	for (i=1; i<=npatch ;i++ ) S1[t][i] = S0[t][i];
	 }
	 for (i=1;i<=npatch ;i++ ) {
  	 for (j=1;j<=npatch ;j++ ) ADa1[i][j] = ADa0[i][j];
	 }
	 j = 0; 
	 for (i=1;i<=npatch ;i++ )if (Xs[-L][i]) j++;

	fprintf(mon, "state X(_L) \n" );
	fprintf(mon, "number occupied at X(_L) = %5d\n",  j);
	 fprintfimatrix(mon, Xs,-L,-L,1,npatch, "%2d");

	
	 meanlogS = 0.0;
	 for (t=1;t<=nyear ;t++ ) {
		 for (i=1;i<=npatch ;i++ )
		   meanlogS += log(S0[t][i]+EPS);
	 }
	 meanlogS /=  npatch*nyear;
	 fprintf(out,"\n mean(log(A)) = %10.3lf   mean(log(S)) = %10.3lf \n", meanlogA, meanlogS);
	fflush(out);

 

  logprior0  = flogprior(hyper_theta,  nhpar, trans_ptheta0,  nyear,  L,  npar,  nxpar,\
 		 theta_lower, theta_upper, trans_theta_prior);
	fprintf(out, "\n likelihood after initialization = %lf \n", loglik0);

/*	Eloglik0 = -FLT_MAX;
	E2loglik0 = 0;
*/

/* end of initialization */


  print_time(stderr);   print_time(out);
			# if  QUICK>=2

			 loglik0 = MC_patches(ini_burn, nyear, npatch, Xs, Xs_tot, L, StartPattern,ti_missinglist, S0, ADa0,\
			 					 logA, logE1_E, ptheta0, npar, X, nxpar, logtrprob0, &Eloglik0, &E2loglik0, startyear, endyear, 
								 propose_t, propose_t1, &accept_p, pab);
			  
			# if DEBUG == 0 || DEBUG == 111
			/* check : are  transitions probabilities after updating consistent 
			         with   transitions probabilities calculated from scratch
			*/
						calc_ADa(ADa0, npatch, A, Dist, Barriers, ptheta0[0][indexalpha], ptheta0[0][indexb], With_area_effect); 
						for (t =-L;t<=nyear ;t++ ) calc_S2( S0[t], npatch, Xs[t], ADa0);

						if ((nxpar[0]==0&&nxpar[1]==0)||(nxpar[0]==1&&nxpar[1]==1)) {
							ptheta0[0][1] = exp(ptheta0[0][1] );
							ptheta0[0][3] = exp(-ptheta0[0][3] );
	  	      	calc_trprob(npatch, -L+1, nyear, Xs, S0, A, ptheta0, npar, X, nxpar, trprob);
							ptheta0[0][1] = log(ptheta0[0][1] );
							ptheta0[0][3] = -log(ptheta0[0][3] );
							for (t=-L+1; t<=nyear; t++) for (i=1; i<=npatch; i++) trprob[t][i] =	log(trprob[t][i]); 	 
						}
						else {
	            calc_logtrprob(npatch, -L+1, nyear, Xs, S0, logA, logE1_E, ptheta0, npar, X, nxpar, trprob);
						}
						for (t=-L+1; t<=nyear; t++) {
							for (i=1; i<=npatch; i++) {
								if ( fabs(trprob[t][i] - logtrprob0[t][i]) > 1.0E-6  )
								 {
               		fprintf(mon," error 1 in logtrprob t = %d i = %d \n", t, i);
               		fprintf(mon,"  logtrprob  =  %10.3lf log(trprob) = %10.3lf\n",
									 logtrprob0[t][i] ,trprob[t][i] );
								}
							}
						}
	 /*
	  fprintfdmatrix(mon, logtrprob0, 3, 4, 6, 10, " %10.3lf " );
 		fprintfimatrix(mon, Xs, 3, 4, 6, 10, " %3d " );
 		fprintfdmatrix(mon, S0, 3, 4, 6, 10, " %10.3lf " );
	 */

			# endif

			#elif QUICK==1
			/* version with saving information within times */
			 loglik0 = Gibbs_patches2(ini_burn, nyear, npatch, Xs, L, StartPattern, ti_missinglist, S0, ADa0,\
			 					 A, ptheta0, npar, X, nxpar, trprob, &Eloglik0, &E2loglik0, startyear, endyear);
			 /*loglik0 = floglik2(npatch, -L+1, nyear, trprob); */
			#else
			for (k = 1; k<=ini_burn ; k++) {
				Gibbs_patches(nyear, npatch, Xs, L, StartPattern, ti_missinglist, S0, Dist, indexalpha,\
	 				 A, ptheta0, npar, X, nxpar, trprob[-L]);
			}	  

 		  loglik0 = floglik(npatch, startyear, endyear, Xs, S0, A, ptheta0, npar, X, nxpar);

			# endif

		 if (Stochastic_EM==1){
				loglik0 = Eloglik0;
		 }


		acc =0;
		delta = delta0;
		T = T0;

/* main loop of Metropolis Hasting iterations */
	for (iter=1;iter<=niter ;iter++ ) {
		
 	if  (Stochastic_EM == 1){
		delta = delta0 * exp(-(mu*iter)/niter);
		T = T0 * exp(-(mu*iter)/niter);
	}
		for (iter_theta=1; iter_theta<=nsubitertheta ; iter_theta++) {
		
		/* generate a new proposal theta1 from trans_theta0 + random values */

		
		propose(trans_ptheta1, trans_ptheta0, npar, \
					to_be_estimated, nestimate, chol_factor, delta, ranvector);
		
		# if QUICK>=2
   	  fbacktrans_ptheta2(ptheta1, trans_ptheta1, npar, trans_par);    
		#else
   	  fbacktrans_ptheta(ptheta1, trans_ptheta1, npar, trans_par);    
		# endif
		logprior1  = flogprior(hyper_theta,  nhpar, trans_ptheta1,  nyear,  L,  npar,  nxpar,\
 	   	 theta_lower, theta_upper, trans_theta_prior);

		if (logprior1 > -FLT_MAX) {
			/* the new proposal is within the specified bounds */
			/* if new alpha (=ptheta[][indexalpha]), calculate S from scrath */

			if (estimate_alpha) { /* S1 changes becuae of alpha */
				# if QUICK
						calc_ADa(ADa1, npatch, A, Dist, Barriers, ptheta1[0][indexalpha], ptheta1[0][indexb], With_area_effect); 
						for (t =-L;t<=nyear ;t++ ) calc_S2( S1[t], npatch, Xs[t], ADa1);
				#else
					 	for (t =-L;t<=nyear ;t++ ) calc_S( S1[t], npatch , Xs[t], A, Dist, ptheta1[t][indexalpha]);
				# endif
			}
			else { /* ADa unchanged, thus S1 = S0 ;Beware: S0 points to S of a previous Xs  */
				dmat_pointer = S1; S1 = S0;	S0 = dmat_pointer;  
			}

			/* Calculate the likelihood of ptheta1 for the current Xs */

			 #if !REGIONAL_STOCHAST
			  for (i=1;i<=npatch ;i++ ) {
    			lp = 0.0;
		  		for (j= 1;j<= nxpar[0] ;j++ )  lp +=  ptheta1[0][nmpar+j]*X[i][j];
			    logE = DMIN(ptheta1[0][1] - ptheta1[0][2] * logA[i] + lp, log1_EPS); 
				  /* logE = DMIN(ptheta1[0][1] - ptheta1[0][2] * logA[i], log1_EPS); */
			    logE1_E[i][0] = logE = DMAX(logE, logEPS);
					logE1_E[i][1]= log(1.0 - exp(logE)); /* log(1-E) */
				}
			 #endif

	    calc_logtrprob(npatch, -L+1, nyear, Xs, S1, logA, logE1_E, ptheta1, npar, X, nxpar, logtrprob1);
    if (Use_J_model) {
		   /* Use Hanski' s incidence model for time 1 
		   save the logtransitions for time 1  and set J in column one*/
		   swap_pd(&(logtrprob1[propose_t]),&(logtrprob1[1]));
			 for (i=1;i<=npatch ;i++ ) {
			   logE = logptrans( 0, 1, S1[1][i], logA[i], logE1_E[i], ptheta1[0], npar, X[i], nxpar);
			   logC = logptrans( 1, 0, S1[1][i], logA[i], logE1_E[i], ptheta1[0], npar, X[i], nxpar);
				 if (Xs[1][i]) {
				 /* log J */
				   logtrprob1[1][i] = -log(1. + exp(logE - logC));
				 }
				 else {
				 /* log (1-J) */
				   logtrprob1[1][i] = -log(1. + exp(logC - logE));
				 }
			 }
		} /* end Use_J-model */

    loglik1 = floglik0(npatch, startyear, endyear, logtrprob1);


    if (Use_J_model) swap_pd(&(logtrprob1[propose_t]),&(logtrprob1[1]));
		 /*loglik1 ready */

		 /*
			myfwrite(mon, iter, lag, Eloglik0, sqrt(E2loglik0), ptheta1, trans_ptheta1, 0, accept_p);
			myfwrite(mon, iter, lag, loglik1, logprior1, ptheta1, trans_ptheta1, npar, accept_p);
		 */
 	
		 if (Stochastic_EM==1) loglik1 = Eloglik1;
		

		 if (Eloglikmax < loglik1) {
		 	/* save maximum likelihood estimate */
		 	 Eloglikmax = loglik1;
			 copy_dvector(ptheta1[0],theta_opt,npar);
		 }
	
	/* compare the new posterior (logprior + loglikelihood) with the previous value
				log r is the log of the ratio of densities p(proposal theta)/ p(theta)
	*/
			logr = (T*takepri*logprior1 + loglik1)-(T*takepri*logprior0 + loglik0) ;

			if ( logr > T*log(urand(&seed)) ) {
				/* accept proposal */
				copy_theta_prop(nyear, L, npar, ptheta1, ptheta0);
				copy_theta_prop(nyear, L, npar, trans_ptheta1, trans_ptheta0);
				
				loglik0 = loglik1;
				logprior0= logprior1;
				dmat_pointer = ADa0; ADa0 = ADa1;	ADa1 = dmat_pointer;  
 				dmat_pointer = S0; S0 = S1;	S1 = dmat_pointer;  
				dmat_pointer = logtrprob0; logtrprob0 = logtrprob1;	logtrprob1 = dmat_pointer;  
		

				acc += 1;
/*				fprintf(mon, " accepted\n");*/
			}
			else {
			/*  no accept ; swap S0 back if !alpha_estimate*/
 				if (!estimate_alpha){dmat_pointer = S0; S0 = S1;	S1 = dmat_pointer;}  
			}
		} /* end logprior >= -FLT_MAX */

/* write iter, loglik0, theta0, trans_theta0  */
			myfwrite(out, iter, lag, loglik0, logprior0, ptheta0, trans_ptheta0, npar, accept_p);
			myfwrite2(mon, iter, lag, Xs, ti_missinglist,nyear, indexnyear);


		 }/*end of for iter_theta*/

			/* update missing patch states for accepted ptheta, which is now called theta0
			 and calculate the likelihood for theta0 */

			# if  QUICK >=2
			 loglik0 = MC_patches(nsubiterXs, nyear, npatch, Xs, Xs_tot, L, StartPattern,ti_missinglist, S0, ADa0,\
			 					 logA, logE1_E, ptheta0, npar, X, nxpar, logtrprob0, &Eloglik0, &E2loglik0, startyear, endyear, 
								 propose_t, propose_t1, &accept_p, pab);

			# if DEBUG == 111
			/* check : are  transitions probabilities after updating consistent 
			         with   transitions probabilities calculated from scratch
			*/
						calc_ADa(ADa0, npatch, A, Dist, Barriers, ptheta0[0][indexalpha], ptheta0[0][indexb], With_area_effect); 
						for (t =-L;t<=nyear ;t++ ) calc_S2( S0[t], npatch, Xs[t], ADa0);
						if ((nxpar[0]==0&&nxpar[1]==0)||(nxpar[0]==1&&nxpar[1]==1)) {
							ptheta0[0][1] = exp(ptheta0[0][1] );
							ptheta0[0][3] = exp(-ptheta0[0][3] );
	  	      	calc_trprob(npatch, -L+1, nyear, Xs, S0, A, ptheta0, npar, X, nxpar, trprob);
							ptheta0[0][1] = log(ptheta0[0][1] );
							ptheta0[0][3] = -log(ptheta0[0][3] );
							for (t=-L+1; t<=nyear; t++) for (i=1; i<=npatch; i++) trprob[t][i] =	log(trprob[t][i]); 	 
						}
						else {
	            calc_logtrprob(npatch, -L+1, nyear, Xs, S0, logA, logE1_E, ptheta0, npar, X, nxpar, trprob);
						}
						for (t=-L+1; t<=nyear; t++) {
							for (i=1; i<=npatch; i++) {
								if ( fabs(trprob[t][i] - logtrprob0[t][i]) > 1.0E-6  )
								 {
               		fprintf(mon," error 2 in logtrprob t = %d i = %d \n", t, i);
               		fprintf(mon,"  logtrprob  =  %10.3lf log(trprob) = %10.3lf\n",
									 logtrprob0[t][i] ,trprob[t][i] );
								}
							}
						}
			# endif
	
			#elif QUICK == 1
			 /* version with saving information within times */
			 loglik1 = Gibbs_patches2(nsubiterXs, nyear, npatch, Xs, L, StartPattern, ti_missinglist, S, ADa,\
			 					  A, ptheta1, npar, X, nxpar, trprob, &Eloglik1, &E2loglik1, startyear, endyear);
			 /*loglik1 = floglik2(npatch, -L+1, nyear, trprob); */
			#else
			for (k = 1; k<=nsubiterXs ; k++) {
				Gibbs_patches(nyear, npatch, Xs, L, StartPattern, ti_missinglist, S, Dist, indexalpha,\
	 				 A, ptheta1, npar, X, nxpar, trprob[-L]);
		  }
	 		loglik1 	 = floglik(npatch, startyear, endyear, Xs, S, A, ptheta1, npar, X, nxpar);

			# endif
	
	} /* end of iter main loop */


	fprintf(out, "\n accepted = %d total tried = %d \n", acc, niter);

/*
	fprintf(out, "\n maximum of likelihood = %lf \n ML-estimates:\n", Eloglikmax);

	fprintfdvector(out, theta_opt, 1, npar,  " %lf ");
	#if QUICK >= 2
	  theta_opt[1] = exp(theta_opt[1]);
	  theta_opt[3] = exp(-theta_opt[3]);
	fprintf(out, "\n maximum of likelihood = %lf \n ML-estimates (original scale):\n", Eloglikmax);
 	fprintfdvector(out, theta_opt, 1, npar,  " %lf ");
		#endif
*/
			# if DEBUG == 0 || DEBUG == 111
			/* check : are  transitions probabilities after updating consistent 
			         with   transitions probabilities calculated from scratch
			*/
			if ( logprior1 > -FLT_MAX){
						calc_ADa(ADa0, npatch, A, Dist, Barriers, ptheta0[0][indexalpha], ptheta0[0][indexb], With_area_effect); 
						for (t =-L;t<=nyear ;t++ ) calc_S2( S0[t], npatch, Xs[t], ADa0);
						if ((nxpar[0]==0&&nxpar[1]==0)||(nxpar[0]==1&&nxpar[1]==1)) {
							ptheta0[0][1] = exp(ptheta0[0][1] );
							ptheta0[0][3] = exp(-ptheta0[0][3] );
	  	      	calc_trprob(npatch, -L+1, nyear, Xs, S0, A, ptheta0, npar, X, nxpar, trprob);
							ptheta0[0][1] = log(ptheta0[0][1] );
							ptheta0[0][3] = -log(ptheta0[0][3] );
							for (t=-L+1; t<=nyear; t++) for (i=1; i<=npatch; i++) trprob[t][i] =	log(trprob[t][i]); 	 
						}
						else {
	            calc_logtrprob(npatch, -L+1, nyear, Xs, S0, logA, logE1_E, ptheta0, npar, X, nxpar, trprob);
						}
						for (t=-L+1; t<=nyear; t++) {
							for (i=1; i<=npatch; i++) {
								if ( fabs(trprob[t][i] - logtrprob0[t][i]) > 1.0E-6  )
								 {
               		fprintf(mon," error 3 in logtrprob t = %d i = %d \n", t, i);
               		fprintf(mon,"  logtrprob  =  %10.3lf log(trprob) = %10.3lf\n",
									 logtrprob0[t][i] ,trprob[t][i] );
								}
							}
						}
					}
		# endif




/*
  for (t =1;t<=nyear ;t++ ) {
   	for (i=1;i<=npatch ;i++ ){

    }
  }

*/

   print_time(stderr);   print_time(out);

/* freeing data vectors and matrices */
   free_imatrix(idata,1,nyear,1,npatch);
   free_ivector(Xs_tot,1,nyear);
   free_state_matrix(Xs,-L3,nyear,1,npatch);
   free_dvector(xcoor,1,npatch);
   free_dvector(ycoor,1,npatch);
   free_dvector(A,1,npatch);
   free_dvector(logA,1,npatch);
	 free_dvector(hyper_theta,0,nhpar);

   free_dmatrix(Dist,1,npatch,1,npatch);
   free_dmatrix(Barriers,1,npatch,1,npatch);
   free_dmatrix(ADa0,1,npatch,1,npatch);
   free_dmatrix(ADa1,1,npatch,1,npatch);
   free_dmatrix(S0,-L3,nyear,1,npatch);
   free_dmatrix(S1,-L3,nyear,1,npatch);
   free_dmatrix(X,1,npatch,0,nxparEC);
   free_dmatrix(theta0,-L,nyear,1,npar);
   free_dmatrix(theta1,-L,nyear,1,npar);
   free_dmatrix(trans_theta0,-L,nyear,1,npar);
   free_dmatrix(trans_theta1,-L,nyear,1,npar);
   free_pdmatrix(ptheta0,-L,nyear,1,npar);
   free_pdmatrix(ptheta1,-L,nyear,1,npar);
   free_pdmatrix(trans_ptheta0,-L,nyear,1,npar);
   free_pdmatrix(trans_ptheta1,-L,nyear,1,npar);


   free_imatrix(ti_missinglist ,1, nmissing+1,1,2);
   free_ivector(to_be_estimated ,1, nestimate+1);
	 free_dvector(theta_ini,1,npar);
	 free_dvector(theta_opt,1,npar);
	 free_dvector(theta_ini_se,1,npar);
   free_dmatrix(trprob,-L3,nyear,1,npatch);
   free_dmatrix(logtrprob0,-L3,nyear,1,npatch);
   free_dmatrix(logtrprob1,-L3,nyear,1,npatch);
	 free_dvector(theta_lower,1,npar);
	 free_dvector(theta_upper,1,npar);
	 free_dvector(trans_par,1,npar);
	 free_dmatrix(trans_theta_prior,1,npar,1,3);

   free_dmatrix(cov_ini, 1, nestimate, 1, nestimate);
	 free_dmatrix(cov0, 1, nestimate, 1, nestimate);
	 free_dmatrix(cov1, 1, nestimate, 1, nestimate);
	 free_dmatrix(chol_factor, 1, nestimate, 1, nestimate);
	 free_dvector(ranvector, 1, nestimate);
	 free_dvector(theta_mean, 1,nestimate);
	 free_dvector(theta_se, 1,nestimate);
	 free_dmatrix(theta_cov, 1, nestimate, 1, nestimate);
	 free_dmatrix(logE1_E,1,npatch,0,1);
	 free_dmatrix(pab,1,npatch,0,3);



   fprintf(stderr, "\nready.\n");
   fclose(out); fclose(mon); 
   # if CHECK
   	 fclose(dat);
   	 fclose(par);
	 # endif
	 if(read_Dij) fclose(dis);
   return 0;
}

