/* fmetapop.h: header file for baysian fitting the patch occupancy model of
   metapopulation dynamics
   C.J.F. ter Braak, CBW, Wageningen
   c.j.f.terbraak@plant.wag-ur.nl
   June 27, 2000 */


#ifndef _FMETAPOP_H

#define _FMETAPOP_H

#define FMETAVERSION  0.8


#include <stdio.h>
#include <stdlib.h>

#ifndef EXIT_FAILURE              /* not defined in pre-ansi versions */
#  define EXIT_FAILURE 1
#endif
#ifndef EXIT_SUCCESS
#  define EXIT_SUCCESS 0
#endif
#ifndef ERREX
#  define ERREX   EXIT_FAILURE    /* parameter value for exit() call at abnormal program termination */
#endif
#ifndef SUCEX
#  define SUCEX   EXIT_SUCCESS    /* parameter value for exit() call at   normal program termination */
#endif

#define urand(plong) ran1(plong)          /* uniform(0,1) */


/*=== typedefs: ===*/
#define DEBUG (0)
#define	CHECK (0)    /* 1 = reading random number from file ; 0 = default*/
#define ZEROONE (1)   /* other values of zeroone then one lead to nonsense.. */
#define MISSING (-1)
#define HUGE (99999.9)
#define IHUGE (32767)
#define QUICK (2)	/*
 QUICK = 0  Gibbs , original version		(does not work any more)
 QUICK = 1 reuse information as much as possible to speed Gibbspatches (does not work any more)
 QUICK = 2 MH-updating per year based on S(t), as described in the Ecology 2002 paper
 QUICK = 3 MH-updating per year based on S(t-1), independent of current year data
 if QUICK >=2 the parameters e and y2 are transformed to logs during initialization
    e  ->  log(e)
		y2 -> -log(y2)
  */

		/* numerical cut-off of log probabilities in logptrans and MC_patches */
#define	EPS (1.0E-15)


#define REGIONAL_STOCHAST 0 /* if 1 some modelparameters are random across years*/
#define WITH_AREA_EFFECT 0   /* connectivity S calculated with or without Area effec*/
#define WITH_RESCUE_EFFECT 1 /* probability of extinction calculated as (1-c)* E or E*/

#if ZEROONE
typedef int   state_t;     /* state of patch is 0/1 (int)  */

#define  state_matrix(a,b,c,d) imatrix(a,b,c,d)
#define  free_state_matrix(a,b,c,d,e)    free_imatrix(a,b,c,d,e)
#define  state_vector(a,b)     ivector(a,b)
#define  free_state_vector(a,b,c) free_ivector(a,b,c)


#else

typedef double   state_t;     /* state of patch is  prob. (double) */

#define  state_matrix(a,b,c,d) dmatrix(a,b,c,d)
#define  free_state_matrix(a,b,c,d,e)    free_dmatrix(a,b,c,d,e)
#define  state_vector(a,b)     dvector(a,b)
#define  free_state_vector(a,b,c) free_dvector(a,b,c)

#endif  /* ZEROONE */

/* utility functions*/

float smpos(float x, float scale);
int any(int n1, int n2, long* ps);
void error(char why[]);
void message(char why[]);
void meta_message(FILE *out, char why[]);
void print_time(FILE *out);
void readf(FILE *in, FILE *out, char* identifier, int npar, double* x, double minx, double maxx, \
                    double* y, double miny, double maxy);

void readi(FILE *in, FILE *out, char* identifier, int npar, int* x, int minx, int maxx, \
                                       int* y, int miny, int maxy);
void readl(FILE *in, FILE *out, char* identifier, long npar, long* x, long minx, long maxx, \
                                       long* y, long miny, long maxy);

int read_file(int npatch, int nyear, double *xcoor, double *ycoor, double *A, int **idata, \
                    int nxpar, double  **X);


/* model functions*/

double ptrans(state_t x1i, state_t x0i, double S0, double A,\
              double theta0[], int npar, double X[],int *nxpar);
double floglik(int npatch, int startyear, int endyear, state_t **Xs, \
  double **S, double *A, double **theta, int npar, double **X, int *nxpar);

double flogprior(double *hypertheta, int nhpar,\
		 double **trans_theta, int nyear, int L, int npar, int *nxpar,\
 		 double *trans_theta_lower, double *trans_theta_upper, double **trans_theta_prior);
int **create_missinglist(int r, int c, int **idata, int *nmissing);
void calc_D(int npatch, double *xcoor, double *ycoor, double **Dist);

void calc_trprob(int npatch, int startyear, int endyear, state_t **Xs, \
 		double **S, double *A, double **theta, int npar, double **X, int *nxpar, double **trprob);

int update_mean_sumss(long n, double sample, double *sammean, double *samssqr);



void copy_theta_ini(int nyear, int L, int npar, double *theta_ini, double **theta, double **ptheta);
void copy_theta_prop(int nyear, int L, int npar,\
	 double **theta_prop, double **theta);
void	propose(double **trans_theta1, double **trans_theta0, int npar, \
			int *to_be_estimated, int nestimate, double **chol_factor, double delta, double *ur);

double randnormal (void);


void logo_fmetapop(FILE *out);

#if QUICK == 0 || QUICK== 1
void ftrans_theta(double *theta, double *trans_theta, int npar, double *trans_par);
void fbacktrans_theta(double *theta, double *trans_theta, int npar, double *trans_par);
void ftrans_ptheta(double **theta, double **trans_theta, int npar, double *trans_par);
void fbacktrans_ptheta(double **theta, double **trans_theta, int npar, double *trans_par);
#endif /*  QUICK == 0 || QUICK== 1 */


#if QUICK == 0

double calc_Si(int i, int npatch, state_t *Xs, double *A, double *Dist, double alpha);
void calc_S(double *S, int npatch, state_t *Xs, double *A, double **Dist, double alpha);
double c_odds(int npatch, int i, state_t x0i, state_t x1[], state_t x2[], \
  double S0i, double S1[], double A[], \
  double Di[], double theta1[], double theta2[], int npar, double** X,int *nxpar);
state_t upd_state_and_S(int npatch, int i, double probti,\
 state_t xtiold, double *St, double *A, double *Di, double alpha);
void init_Xs_S(int nyear, int npatch, int **idata, int **ti_missinglist, int nmissing, \
 state_t **Xs, int L, int StartPattern, double **S, double **D, int indexalpha,\
 double *A, double *theta_ini, int npar, double **X, int *nxpar, double f_ini);
void Gibbs_patches(int nyear, int npatch, \
 state_t **Xs, int L, int StartPattern, int **ti_missinglist, double **S, double **D, int indexalpha,\
 double *A, double **ptheta, int npar, double **X, int *nxpar, double *trprob);


#endif /* QUICK== 0*/

#if QUICK == 1

double Gibbs_patches2(long nsubiter, int nyear, int npatch, \
 state_t **Xs, int L, int StartPattern, int **ti_missinglist, double **S, double **ADa,\
 double *A, double **ptheta, int npar, double **X, int *nxpar, double **trprob,\
 double *pEloglik, double *pE2loglik, int startyear, int endyear);
 

double c_odds2(int npatch, int i, int t, state_t x0i, state_t x1[], state_t x2[], double S0i,\
   double S1[], double A[], double ADai[], \
  double theta1[], double theta2[], int npar, double** X, int *nxpar,\
	 double *plprob, double *plprob_other, double *pprobti);

state_t upd_state_and_S2(int npatch, int i, double probti, state_t xtiold,\
      double *St, double *ADai, \
      double *plprob, double lprob_other, double trprobti, double *newtrprobti, int *state_change);

double floglik2(int npatch, int startyear, int endyear, double **trprob);
void init_Xs_S1(int nyear, int npatch, int **idata, int **ti_missinglist, int nmissing, \
 state_t **Xs, int L, int StartPattern, double **S, double **ADa,\
 double *A, double *theta_ini, int npar, double **X, int *nxpar, double f_ini);
 

#endif /* QUICK== 1*/

/* for one-based  double matrices and vectors  */
void swap_pi(int **p0, int **p1);
void swap_pd(double **p0, double **p1);
int sum_row_imatrix(int r, int c, int **imat, int missing, int sum[]);


void transpose_dmatrix(double **a, double **at, int nr, int nc );
void multiply_dmatrices ( double **a, double **b, double **ab,
                        int rows_a, int cols_a, int cols_b );
void  multiply_dmat_const( double **a, double c, double **ca, int rows_a, int cols_a);

void mult_dmatvec ( double **m, double *v, double *mv, int rows, int cols );
void mult_dlowermatvec ( double **m, double *v, double *mv, int rows);

void copy_dmatrix ( double **source, double **dest, int rows, int cols );
void copy_dvector ( double *source, double *dest, int n );

/* Given a positive-definiet symmetric matrix s[1...n][1...n] this routine constructs
   its Choleski decompositon s = l l' 
	 From the input s its upper triangle is used, diagonal inclusive
	 The choleski factor l is stored in lower triangular form, diagonal inclusive
	 The upper triangle is set to 0, as required. 
	 If s is not positive definite an error and 0 is returned
 	*/
int  cholesky(double** s, double** l, int n);
void read_lower_matrix(FILE *in, double **cov, int p);
/* sets the square matrix cov to diag(diag_values) */
void set_cov_diag(double **cov, int p, double *diag_values);
/*  creates a vector-index of length ncrit
		 indicating elements of a double vector x that are equal to the value  crit
	   ending list with a zero 
		 The values returned assume that x is x[1...n] */
int *create_vindex(int n, double *x, int *ncrit, double crit);
	
#endif /* _FMETAPOP_H */

