/* functions metapop  */ 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <limits.h>
#include <nrutil.h>
#include <nr.h>


#include  "fmetapop.h"
#include  "MC_patches.h" 

extern FILE *out, *par, *dat, *mon;
extern long seed;
extern int Stochastic_EM;

# if QUICK==1
  /* time__2 used in c_odds2 and Gibbs_patches2 */
  static int time__2=-10000;
#endif
double ptrans(state_t x1i, state_t x0i, double S0, double A, \
              double theta0[], int npar, double X[], int *nxpar)
/* ptrans calculates the transition probability
   to state x1i of patch i at time 1,
   for a given patch (i) given state x01 at time 0 (of patch 1) 
   and the connectivity value S0 of the patch from time 0, its area A,
   extra explanatory variables in X[], 
   and the npar-vector of parametervalues  theta0 at time 0
   x1i = state at time 1 of patch i
   x0i = state at time 0 of patch i
   The state can be  0 or i or a value between 0 and 1
   
   */
{
/*
 static double EPS = 1.0E-15;
 */
 #define ONE 1.00000
 int j, beginE, beginC;
 double prob, C, E,  e, x, y, z;
/*
  e = theta0[1];
  x = theta0[2];
  y = theta0[3];
  z = theta0[4];
  alpha = theta0[5]
	*/
 
  #if STATE == ZER0ONE
	 /* indexes in theta for the X variables for E model and C model */
	 beginE = npar - nxpar[0] - nxpar[1];
	 beginC = beginE + nxpar[0];

   /*  state_t == int */
      if (x0i) {
				/* extinction probabilities*/
				 e = theta0[1];
				 x = theta0[2];
				 if (nxpar[0]==1&&nxpar[1]==1) {
		  		 E = e * exp(X[1]*theta0[7]) /  pow(A,x)  ;  
				 }
				 else {
		  		 E = e  /  pow(A,x)  ;  
				 }
			   E = DMIN(E, (double) ONE);
			   #if WITH_RESCUE_EFFECT
      		  y = theta0[3];
		    		z = theta0[4];
	  			  if (S0 < EPS) C = 0.0; else {
				       if (nxpar[0]==1&&nxpar[1]==1) {
						     C = ONE/ ( ONE +  y / (pow(S0, z)*exp(X[2] * theta0[8]) ));
							 }
							 else {
						     C = ONE/ ( ONE +  y / pow(S0, z) );
							 }
						}
			      E  *= (ONE - C);
			   #endif
			   if (x1i) prob = ONE - E;  else prob = E;
      }
      else {
				 /* colonization probabilities*/
  		   y = theta0[3];
				 z = theta0[4];
	  		 if (S0 < EPS) C = 0.0; else {
				       if (nxpar[0]==1&&nxpar[1]==1) {
						     C = ONE/ ( ONE +  y / (pow(S0, z)*exp(X[2] * theta0[8]) ));
							 }
							 else {
						     C = ONE/ ( ONE +  y / pow(S0, z) );
							 }
				 }
	       if (x1i) prob = C;      else prob = ONE - C;
      }
      
  #else
   /* state_t == double */ 
      {double logprob;
  e = theta0[1];
  x = theta0[2];
  y = theta0[3];
  z = theta0[4];
  /* colonization prob is always needed for rescue effect */
   if (S0 < EPS) C = 0.0;
   else
   C = ONE/ ( ONE +  y / pow(S0, z) );
  
   if (x0i > EPS  ) {
 /* look at extinction events ( x0i != 0)  */
   E = e / pow(A,x);  
   E = DMIN(E, (double) ONE);
   #if WITH_RESCUE_EFFECT
      E  *= (ONE - C);
   #endif
   } 

      logprob = x0i * (x1i * log(ONE-E)  + (ONE-x1i)* log(E)) +
               (ONE-x0i)* (x1i * log(C)  + (ONE-x1i)* log(ONE-C));
      prob = exp(logprob);
      }
  #endif
  /* avoid numerical trouble with prob = 0 and 1 - prob = 0*/
  prob = DMAX(prob, EPS);
  prob = DMIN(prob, ONE-EPS);

  return prob;
}




double floglik(int npatch, int startyear, int endyear, state_t **Xs, \
	double **S, double *A, double **theta, int npar, double **X, int *nxpar)
/* loglik calculates the loglikelihood of the model 
		from years startyear to and including endyear
		using calls to function ptrans	of transition probabilities trprob

   state_t  **Xs;  states with front times and imputed missings 
   double *A,       vector of Areas  
	        **S,      times x patches matrix of connectivities   
          **X,      patches x nvars matrix of extra explanatory variables  
          **theta,  times x npar matrix of parameter values  
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram
startyear and endyear must therefore lie in the range -L , nyear

		
*/
{ 
	int t,i;
	double llik;

	llik = 0;

	  for (t =startyear;t<=endyear ;t++ ) {
   		for (i=1;i<=npatch ;i++ ){
    		llik += \
				log(ptrans(Xs[t][i], Xs[t-1][i], S[t-1][i], A[i], theta[t], npar,\
					 X[i], nxpar));
				
			
			}
	 }
	 return llik;
}

double flogprior(double *hypertheta, int nhpar,\
		 double **trans_theta, int nyear, int L, int npar, int *nxpar,\
 		 double *trans_theta_lower, double *trans_theta_upper, double **trans_theta_prior)
/* calculates the log of the prior for trans_theta[][]
   using lower and upperbounds for thrans_theta
	 and a normal prior with mean trans_theta_prior[k][1] 
	  and standard deviation trans_theta_prior[k][2] for the k-th parameter
*/
{
	int t,k;
	double logprior;
	#if !REGIONAL_STOCHAST
	/* check lower and upper bounds */
	for (k=1;k<=npar ; k++ ) {
		if (trans_theta[0][k] < trans_theta_lower[k] || trans_theta[0][k] > trans_theta_upper[k]) {
			return -FLT_MAX;
		}
	}
	/* trans_theta is within the specified bounds;
	   now calculate the log of an unnormalized normal prior */
	for (logprior = 0.0, k=1;k<=npar ; k++ ) {
		logprior += DSQR( (trans_theta[0][k] - trans_theta_prior[k][1])/trans_theta_prior[k][2] );
	}
	#else

	meta_message(out, " *** BEWARE check logprior of regional stochast hierarchical model*** ");

	/* check lower and upper bounds */
	for (t = -L+1; t<=nyear; t++) {
		for (k=1;k<=npar ; k++ ) {
			if (trans_theta[t][k] < trans_theta_lower[k] || trans_theta[t][k] > trans_theta_upper[k]) {
				return -FLT_MAX;
			}
	}
		}
/* trans_theta is within the specified bounds; now calculate the unnormalized logprior */
	logprior = 0.0;
	for (t = -L+1; t<=nyear; t++) {
		for (k=1;k<=npar ; k++ ) {
			logprior += DSQR( (trans_theta[t][k] - trans_theta_prior[k][1])/trans_theta_prior[k][2] );
		}
	}
	#endif


	return -0.5*logprior;
}

void calc_D(int npatch, double *xcoor, double *ycoor, double **Dist)
	/* calculates npatch x npatch distance matrix Dist 
	from the x and y coordinates xcoor and ycoor */
{
	int i,j;
	for (i=1;i<=npatch ; i++) {
		for (j=1; j<i; j++ ) {
			Dist[i][j]=sqrt(DSQR(xcoor[i]-xcoor[j])+DSQR(ycoor[i]-ycoor[j]));
	    Dist[j][i] = Dist[i][j];
		}
		Dist[i][i] = 0.0;
	}
}

void copy_theta_ini(int nyear, int L, int npar, double *theta_ini, double **theta, double **ptheta)
{
/* copy theta_ini from input to theta and
   assign the rowpointers  ptheta to the rows of theta

 use in calling code ptheta rather than theta!!
*/
int t,k;
#if !REGIONAL_STOCHAST
  for (k=1;k<=npar;k++ ) theta[0][k] = theta_ini[k];
	for (t=-L;t<=nyear ;t++ ) ptheta[t] = theta[0];
#else
  for (t=-L;t<=nyear ;t++ ) {
    for (k=1;k<=npar;k++){
      theta[t][k]= theta_ini[k];
    }
    ptheta[t]= theta[t];
  }
#endif
}
void copy_theta_prop(int nyear, int L, int npar,\
	 double **theta_prop, double **theta)
{
/* copy theta_prop to theta 
   use in calling code ptheta rather than theta (assigned in copy_theta_ini)!!
*/
int t,k;
#if !REGIONAL_STOCHAST
  for (k=1;k<=npar;k++ ) theta[0][k] = theta_prop[0][k];
#else
  for (t=-L;t<=nyear ;t++ ) {
    for (k=1;k<=npar;k++){
      theta[t][k]= theta_prop[t][k];
    }
  }
#endif
}



double fbacktrans(int k, double trans_b)
/* function that backtransforms the  parameter b */
{
	double b;
  switch (k) {
	case 1 : b = exp(trans_b);  break;
	case 2 : b = DSQR(trans_b);  break;
  default: b = trans_b;     	break;
	
	}
	return b;
}
 double ftrans(int k, double b)
/* function that transforms the parameter b */
{
	double trans_b;
  switch (k) {
	case 0 : trans_b = log(b);  	break;
	case 2 : trans_b = sqrt(b); 	break;
  default: trans_b = b;      					break;
	
	}
	return trans_b;
}


void	propose(double **trans_theta1, double **trans_theta0, int npar, \
			int *to_be_estimated, int nestimate, double **chol_factor, double delta, double *ur)
/* generate multivariate normal proposals for the parameters on the transformed scale (thrans_theta)
	 using a scaled choleski factor (a lower triangular matrix) derived from the (target) covariance matrix
	 ur is a workspace vector of length nestimate
*/
{
	int j,k;
	double Az;
#if !REGIONAL_STOCHAST
	
	for (k=1; k<=nestimate; k++) {
		ur[k] = randnormal();
		for (Az = 0.0, j=1;j<=k ;j++ )  Az += chol_factor[k][j]* ur[j];
		trans_theta1[0][to_be_estimated[k]] = trans_theta0[0][to_be_estimated[k]] +  delta * Az;
	}
#else

 error( " Random model proposal to be programmed in pro9pose    ");
#endif
	
}

#define M_PI             3.14159265358979323846
#define TWO_PI           6.283185307179586
double randnormal ( void )
/* Box-Muller random normal deviates */
{
  static char chifirst= 1;
  static double chisecond;
  double angle, radius, cosang, sinang;

  if (chifirst)
    {
    radius= sqrt(-2*log(urand(&seed)));
    angle= TWO_PI*urand(&seed);
    cosang= cos(angle);
    sinang= sqrt(1.0-cosang*cosang);
    if (angle>M_PI) sinang= -sinang;
    chisecond= radius*sinang;
    chifirst= 0;
    return  radius*cosang;
    }
  else
    {
    chifirst= 1;
    return  chisecond;
    }

} /* randnormal() */

void calc_trprob(int npatch, int startyear, int endyear, state_t **Xs, \
	double **S, double *A, double **theta, int npar, double **X, int *nxpar, double **trprob)
/* calculates the transition probabilities trprob from startyear to endyear for all patches

   state_t  **Xs;  states with front times and imputed missings 
   double *A,       vector of Areas  
	        **S,      times x patches matrix of connectivities   
          **X,      patches x nvars matrix of extra explanatory variables  
          **theta,  times x npar matrix of parameter values  
          **trprob,times x npatch matrix of transition probabilities
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram
startyear and endyear must therefore lie in the range -L , nyear

If DEBUG
output: 
trprob transition probabilities for t = startyear to endyear (all npatch patches)
			
*/
{ 
	int t,i;
	  for (t =startyear;t<=endyear ;t++ ) {
   		for (i=1;i<=npatch ;i++ )
				trprob[t][i] = ptrans(Xs[t][i], Xs[t-1][i], S[t-1][i], A[i], theta[t], npar,\
					 X[i], nxpar);
	 }
}
int update_mean_sumss(long n, double sample, double *sammean, double *samssqr)
/*  Function for updating the Sample mean and corrected sum of squares
		Applied Statistics Algorithm AS41
		by M.R. Clarke
*/
{
   // beware n starts from 1!!
   double d;

   if (n==1)
   {
      *sammean = sample;
      *samssqr = 0.0;
   }
   else
   {
      d = sample - (*sammean);
      *sammean += d/n;
      *samssqr += (1-(1.0/n))* d*d;
   }     
   return 0;
}       


void logo_fmetapop(FILE *out)
{
 fprintf(out, " Program fmetapop version %.1f \n" 
  " to fit patch dynamics models from incidence and turnover data by Bayesian methods\n", FMETAVERSION);
 fprintf(out," Author: Cajo J.F. ter Braak, Biometris, Wageningen-UR, Box 100 6700 AC Wageningen\n");
 fprintf(out," E-mail: C.J.F.terBraak@plant.wag-ur.nl   See also www.canoco.com \n\n  ");
 fprintf(out," Compiled with header file variables:\n QUICK = %d\n DEBUG = %d\n MISSING = %d\n"
  " REGIONAL_STOCHAST = %d\n WITH_RESCUE_EFFECT = %d\n",QUICK, DEBUG, MISSING, REGIONAL_STOCHAST, WITH_RESCUE_EFFECT);
 
 if (QUICK==0)fprintf(out, " WITH_AREA_EFFECT = %d which can not be overruled in the parameter file\n\n", WITH_AREA_EFFECT);
 fprintf(out," Further parameters are specified in the file name.par,\n"
   " the data in the file name.dat file, with default name fmetapop\n\n");
 

}

#if QUICK == 0 || QUICK== 1
 void ftrans_ptheta(double **ptheta, double **trans_ptheta, int npar, double *trans_par)
/* transforms the parameter matrix ptheta
					 to the paramater matrix trans_ptheta on the linkscale	
					  taking care about REGIONAL_STOCHAST 0/1				 
						*/
								
{
#if !REGIONAL_STOCHAST
   ftrans_theta(ptheta[0], trans_ptheta[0], npar, trans_par);
#else
  for (t=-L;t<=nyear ;t++ ) ftrans_theta(ptheta[t], trans_ptheta[t], npar, trans_par);
  
#endif
}




void fbacktrans_ptheta(double **ptheta, double **trans_ptheta, int npar, double *trans_par)
/* backtransforms the parameter matrix trans_ptheta on the  linkscale
									 to paramater matrix ptheta 			 on the original scale
 taking care about REGIONAL_STOCHAST 0/1
									  
									 */
{
#if !REGIONAL_STOCHAST
   fbacktrans_theta(ptheta[0], trans_ptheta[0], npar, trans_par);
#else
  for (t=-L;t<=nyear ;t++ ) fbacktrans_theta(ptheta[t], trans_ptheta[t], npar, trans_par); 
  
#endif
}

void ftrans_theta(double *theta, double *trans_theta, int npar, double *trans_par)
/* transforms the parameter vector theta
					 to the paramater vector trans_theta on the linkscale*/
{
	int k;

	/* initialize with no transformation */
	for (k=1;k<=npar ;k++ ) trans_theta[k] = theta[k];


	 /* put here transformations*/

	 /*  parameter e   */
	 /* apply a centring to logA, i.e. trans_par[1] = mean(logA)
	       trans_theta1 =  log (e) - x * mean(log(A)) */
	 trans_theta[1] = log(theta[1]) - theta[2]*trans_par[1];

	 /*  parameter y*/
	 /* apply a centring to logS, i.e. trans_par[3] = mean(logS) 
	       trans_theta3 =  log (y2) - z * mean(log(S)) */
	 trans_theta[3] = log(theta[3]) - theta[4]*trans_par[3];


}

void fbacktrans_theta(double *theta, double *trans_theta, int npar, double *trans_par)
/* backtransforms the parameter vector trans_theta on the  linkscale
									 to paramater vector theta 			 on the original scale */
{
	int k;

	/* initialize with no transformation */
	for (k=1;k<=npar ;k++ ) theta[k] = trans_theta[k];


	 /*  parameter e*/
	
	 /*      theta1 =  exp (   trans_theta1 + x * mean(log(A)) ) */
	 theta[1] = exp(trans_theta[1] + theta[2]*trans_par[1]);

	 /*  parameter y*/
	 /*      theta3 =  exp (   trans_theta3 + z * mean(log(S)) ) */
	 theta[3] = exp(trans_theta[3] + theta[4]*trans_par[3]);


}
#endif	 /* QUICK== 0 || QUICK == 1*/
				 

#if QUICK == 0
/* Gibbs version without any speedup */
double c_odds(int npatch, int i, state_t x0i, state_t x1[], state_t x2[], double S0i,\
   double S1[], double A[], double Di[], \
  double theta1[], double theta2[], int npar, double** X, int *nxpar)
/* conditional odds for patch i at time 1 given states at time 0 and time 2 
     odds = prob(x1i=1)/prob(x1i=0)

 **BEWARE: if x1i =0 gives an extinct metapopulation at time 1 then prob(xi=1)= 1,
           Take care of this BEFORE the call to   c_odds 
          
 npatch = number of patches
 x0i    = state of patch i at time 0 
 x1[]= state of patches at time 1
 S0i    = connectivity of patch i for time 1 calculated from states at time 0
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 theta1[] = parameters to calculate probs for time 1
 x2[]   = states at time 2
 S1[]   = vector of connectivities of all patches from (current) states for time 2
 Di[]   = vector of Distances of all patches to patch i
 theta2[] = parameters to calculate probs for time 2
*/
{
  int j;
  double f,probxi1, lprob1, lprob0, S1j, alpha, S_add;
  state_t occupied, empty, x1iold;
  occupied = 1;
  empty = 0;
  x1iold = x1[i];
  alpha = theta2[npar];
  probxi1 = ptrans(occupied, x0i, S0i, A[i], theta1, npar, X[i], nxpar);

#if DEBUG
  {
  state_t stot1;
  stot1 = 0;
  for ( j=1; j<= npatch; j++) stot1 += x1[j];
  if (stot1 - x1[i]<0.000001)  {
    fprintf(out,  "\nError: metapopulation extinct in c_odds");
    exit(0); 
    }
  }
#endif

  /* metapopulation not extinct at time 1 with x1i =0  */
   lprob1 = 0.0;
   lprob0 = 0.0;
   for ( j = 1; j <= npatch; j++){
     if (j == i ) {
 /* BEWARE : only correct for prob states
   if at most one missing in year 1 and  NO missings in year 0 and 2
 */
       lprob1 += log( ptrans(x2[i], occupied, S1[i], A[i], theta2, npar, X[i], nxpar) );
       lprob0 += log( ptrans(x2[i], empty,    S1[i], A[i], theta2, npar, X[i], nxpar) );
     }
     else {
  #if WITH_AREA_EFFECT
    S_add =  A[j]* exp(-Di[j]/alpha);
  #else
    S_add =  exp(-Di[j]/alpha);
  #endif
    S1j = S1[j] + S_add * (occupied - x1iold)  ;
 /* check whether correct for prob states */
     lprob1 += log( ptrans(x2[j], x1[j], S1j, A[j], theta2, npar, X[j], nxpar) );
     S1j = S1[j] + S_add * (empty - x1iold) ;
     lprob0 += log( ptrans(x2[j], x1[j], S1j, A[j], theta2, npar, X[j], nxpar) );
     }
   }
   f = exp(lprob1 - lprob0) * probxi1 / (1.0 - probxi1);
   return  f  ;
}

double calc_Si(int i, int npatch, state_t *Xs, double *A, double *D, double alpha){
/* calculates the connectivity S for patch i for an npatch-vector of states */
	double Si = 0.0;
  int j;
	for (j=1;j<=npatch ;j++ ) {
  #if WITH_AREA_EFFECT
    Si += Xs[j] * A[j]* exp(-D[j]/alpha);
  #else
    Si += Xs[j] * exp(-D[j]/alpha);
  #endif
 	}
  #if WITH_AREA_EFFECT
    Si -= Xs[i] * A[i];
  #else
    Si -= Xs[i];
  #endif

	return  Si;
}

void calc_S(double *S, int npatch, state_t *Xs, double *A, double **D, double alpha)
/* calculates the connectivity S (an npatch vector) for all patches
 from an npatch-vector of states
 Output S!
 */
{
  int i,j;
  double Si;
  for ( i=1;i<=npatch ;i++ ) {
  	Si = 0.0;
  	for ( j=1;j<=npatch ;j++ ) {
      #if WITH_AREA_EFFECT
         Si += Xs[j] * A[j]* exp(-D[i][j]/alpha);
      #else
        Si += Xs[j] * exp(-D[i][j]/alpha);
      #endif
  	}
    #if WITH_AREA_EFFECT
      S[i] = Si - Xs[i] * A[i];
    #else
      S[i] = Si - Xs[i];
    #endif
	}

}




state_t upd_state_and_S(int npatch, int i, double probti, state_t xtiold,\
      double *St, double *A, double *Di, double alpha)
/*  draws a new state for patch i from a given probability and
    updates the connectivity vector St for the corresponding time  t
    from the distances Di from patch i and alpha
*/
{
  int j;
  double S_add;
  state_t xtinew;

	# if ZEROONE
  	xtinew = (probti >urand(&seed) );
	#else
		xtinew = probti;
	# endif

  if (xtinew != xtiold){
    for (j=1; j<=npatch; j++){
      if (j!=i){
    /* update S from dis*/
      #if WITH_AREA_EFFECT
        S_add =  A[j]* exp(-Di[j]/alpha);
      #else
        S_add =  exp(-Di[j]/alpha);
      #endif
	    # if ZEROONE
          if (xtinew) St[j] += S_add; else St[j] -= S_add;
 	    #else
          St[j] += S_add * (xtinew - xtiold);
    	# endif
       }
    }
  }
  return xtinew;
}

void init_Xs_S(int nyear, int npatch, int **idata, int **ti_missinglist, int nmissing, \
 state_t **Xs, int L, int StartPattern, double **S, double **D, int indexalpha,\
 double *A, double *theta_ini, int npar, double **X, int *nxpar, double f_ini)
 /*initialize
   - Xs (the states) from data (idata) and imputes the missings by simple imputation
	 					using the initial modelparameters  theta_ini
	 - S  (the connectivities) from Xs, A, D (distances) and alfa
	 Xs and S are (-L...nyear) by (1...npatch)  
*/	
{
	int t,i,k;
	state_t stot;
	double *trprob, alpha;

  trprob    = dvector(1,npatch);
	alpha     = theta_ini[indexalpha];

  for (t =1;t<=nyear ;t++ )	{for (i=1;i<=npatch ;i++ )	Xs[t][i] =idata[t][i];}


	for (k=1;k<=nmissing;  k++ ) {
/* fill missings in all data year */
    t = ti_missinglist[k][1];
		i = ti_missinglist[k][2];
		# if ZEROONE
		Xs[t][i]= (urand(&seed) < f_ini);
		#else
		Xs[t][i]= urand(&seed);
		# endif
	}

/* impute missings for preyears 0 to -L*/

  for (t =0;t>=-L ;t-- ) {
    calc_S( S[t+1], npatch , Xs[t+1], A, D, alpha);
   	for (i=1;i<=npatch ;i++ ){
        trprob[i] = ptrans( 1, Xs[t+1][i], S[t+1][i], A[i], theta_ini, npar, X[i], nxpar);
        #if !ZEROONE
           Xs[t][i] = trprob[t][i];
        #endif
    }
   #if   ZEROONE
    do {
      stot = 0;
   	  for (i=1;i<=npatch ;i++ ){
        Xs[t][i] = (trprob[i]>urand(&seed));
        stot +=  Xs[t][i];
       }
    } while (stot < npatch/20.0);
    #endif

  }
	/* calculate connectivity S[t] for year t (t= -L, 2 ... nyear) */

  calc_S( S[-L], npatch , Xs[-L], A, D, alpha);
  for (t =2;t<=nyear ;t++ ) calc_S( S[t], npatch , Xs[t], A, D, alpha);
  
  fprintf(out," connect S times x npatch\n");
  fprintfdmatrix(out,S,-L,nyear,1,npatch, "%10.2lf");

  free_dvector(trprob,1,npatch);
}


void Gibbs_patches(int nyear, int npatch, \
 state_t **Xs, int L, int StartPattern, int **ti_missinglist, double **S, double **D, int indexalpha,\
 double *A, double **ptheta, int npar, double **X, int *nxpar, double *trprob)
/* Use Gibbs sampling toupdate all missing patches from pretime -L to last data year nyear

   state_t  **Xs;   states with front times, data and imputed missing data 
   double *A,       vector of Areas  
	        **S,      times x patches matrix of connectivities
					**D,      patches x patches matrix of distances   
          **X,      patches x nvars matrix of extra explanatory variables  
          **ptheta, pointers to rows of the times x npar matrix of parameter values  
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram

Note:
	 *trprob is a npatch vector as work space (transition probabilities at time -L
	 
	if (StartPattern <0)  patches at pretime -L are not updated
	*/
{
 int i,k, kk, t;
 state_t stot;
 double probti, f;

			if (StartPattern >=0) {
	 /* update patches at pretime -L*/
		    t = -L;
		    for (i=1;i<=npatch ;i++ ){
		        trprob[i] = ptrans( 1, Xs[t+1][i], S[t+1][i], A[i], ptheta[-L], npar, X[i], nxpar);
		        #if !ZEROONE
		           Xs[t][i] = trprob[i];
		        #endif
		    }
		   #if   ZEROONE
		    do {
		      stot = 0;
		   	  for (i=1;i<=npatch ;i++ ){
		        Xs[t][i] = (trprob[i]>urand(&seed));
		        stot +=  Xs[t][i];
		       }
		    } while (stot < 5);
		    #endif

		  /*calculate connectivity for time t = -L*/
		    calc_S( S[-L], npatch , Xs[-L], A, D, ptheta[-L][indexalpha]);
			} /* if StartPattern*/


	 /* update patches at pretimes from -L+1 to 0*/

	 for (t=-L+1; t<=0; t++){
	    for (i=1; i<=npatch; i++){
	      f = c_odds(npatch, i, Xs[t-1][i], Xs[t], Xs[t+1],\
	      S[t-1][i], S[t], A, D[i], ptheta[t], ptheta[t+1], npar, X, nxpar);
	      probti= f/(1+f);
	      Xs[t][i] = upd_state_and_S(npatch, i, probti, Xs[t][i], S[t], A, D[i], \
	        ptheta[t][indexalpha]);
# if DEBUG==100
			if (t==0)fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
	    }
	 }


	/* update missings in all data year except last */

		for (k=1;(t = ti_missinglist[k][1])< nyear ;  k++ ) {
	  /*t = ti_missinglist[k][1];*/
			i = ti_missinglist[k][2];
	    f = c_odds(npatch, i, Xs[t-1][i], Xs[t], Xs[t+1],\
	       S[t-1][i], S[t], A, D[i], ptheta[t], ptheta[t+1], npar, X, nxpar);
	    probti= f/(1+f);
	    Xs[t][i] = upd_state_and_S(npatch, i, probti, Xs[t][i], S[t], A, D[i], \
	       ptheta[t][indexalpha]);
# if DEBUG==100
			  fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
		}

	/* update missings in last data year */
		for (kk= k ;(t = ti_missinglist[kk][1])== nyear ;  kk++ ) {
	/*   t = ti_missinglist[k][1];      */
			i = ti_missinglist[kk][2];
	    probti = ptrans( 1, Xs[t-1][i], S[t-1][i], A[i], ptheta[t], npar, X[i], nxpar);
	    Xs[t][i] = upd_state_and_S(npatch, i, probti, Xs[t][i], S[t], A, D[i], \
	       ptheta[t][indexalpha]);
# if DEBUG==100
			  fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
		}


}

# endif   /*QUICK== 0*/

# if QUICK == 1
double c_odds2(int npatch, int i, int t, state_t x0i, state_t x1[], state_t x2[], double S0i,\
   double S1[], double A[], double ADai[], \
  double theta1[], double theta2[], int npar, double** X, int *nxpar,\
	 double *plprob, double *plprob_other, double *pprobti)
/*speedy from of c_odds with saving info of log( prob(Xt2 | Xt1}) 
  in lprob (and lprob_other, the log-probability if x1i changes its state
	if a new time t is encounted lprob and lprob_other are calculated from scratch
	                from lprob1 and lprob0
	if an old time t is encountered, lprob is re_used, and lprob_other is re_calculated 
Also
  prob( xti = 1 | {X0})  is saved in pprobti

*/
/* conditional odds for patch i at time 1 given states at time 0 and time 2 
     odds = prob(x1i=1)/prob(x1i=0)

 **BEWARE: if x1i =0 gives an extinct metapopulation at time 1 then prob(xi=1)= 1,
           Take care of this BEFORE the call to   c_odds 
          
 npatch = number of patches
 i      = patch indicator
 t      = time indicator
 x0i    = state of patch i at time 0 
 x1[]= state of patches at time 1
 S0i    = connectivity of patch i for time 1 calculated from states at time 0
 A[]     = area of patch i
 X[i][j]  =  extra explanatory variable j for patch i  , 
 theta1[] = parameters to calculate probs for time 1
 x2[]   = states at time 2
 S1[]   = vector of connectivities of all patches from (current) states for time 2
 ADai[]   = vector of (Area of patch j) * exp(-Distance of patch j to patch i/alpha)
 theta2[] = parameters to calculate probs for time 2
*/
{
  int j;
  double f,probx1i,  lprob1, lprob0, lprob_other, S1j;
  state_t occupied, empty, x1i, x1i_other;
  occupied = 1;
  empty = 0;
  x1i = x1[i];
  x1i_other = 1-x1i; /* the other state (0/1) of x1[i] */   
  *pprobti = probx1i = ptrans(occupied, x0i, S0i, A[i], theta1, npar, X[i], nxpar);

#if DEBUG
  {
  state_t stot1;
  stot1 = 0;
  for ( j=1; j<= npatch; j++) stot1 += x1[j];
  if (stot1 - x1[i]<0.000001)  {
    fprintf(out,  "\nError: metapopulation extinct in c_odds");
    exit(0); 
    }
  }
#endif

  /* metapopulation not extinct at time 1 with x1i =0  */
   if (t != time__2) {

		 time__2 = t;  
	   lprob1 = 0.0;
	   lprob0 = 0.0;
	   for ( j = 1; j <= npatch; j++){
	     if (j == i ) {
	 /* BEWARE : only correct for prob states
	   if at most one missing in year 1 and  NO missings in year 0 and 2
	 */
	       lprob1 += log( ptrans(x2[i], occupied, S1[i], A[i], theta2, npar, X[i], nxpar) );
	       lprob0 += log( ptrans(x2[i], empty,    S1[i], A[i], theta2, npar, X[i], nxpar) );
	     }
	     else {
	       S1j = S1[j] + ADai[j] * (occupied - x1i)  ;
		     lprob1 += log( ptrans(x2[j], x1[j], S1j, A[j], theta2, npar, X[j], nxpar) );
		     S1j = S1[j] + ADai[j] * (empty - x1i) ;
		     lprob0 += log( ptrans(x2[j], x1[j], S1j, A[j], theta2, npar, X[j], nxpar) );
		     }
	   }
     if (x1i) {
   		*plprob = lprob1;
			*plprob_other = lprob0;
		 }
		 else {
   		*plprob = lprob0;
			*plprob_other = lprob1;
		  }


   }
	 else {
	 /* t == time__2 */
	 /* use lprob from previous calculation; thus calculate only  lprob_other */
	 /*  lprob = *plprob;*/
   	 lprob_other = 0.0;
	   for ( j = 1; j <= npatch; j++){
	     if (j == i ) {
       lprob_other += log( ptrans(x2[i], x1i_other, S1[i], A[i], theta2, npar, X[i], nxpar) );
	     }
	     else {

				#if ZEROONE
		      if (x1i_other) S1j = S1[j] + ADai[j]; else S1j = S1[j] - ADai[j];
				#else
			     S1j = S1[j] + ADai[j] * (x1i_other - x1i)  ;
				# endif
		      lprob_other += log( ptrans(x2[j], x1[j], S1j, A[j], theta2, npar, X[j], nxpar) );
		     }
	   }

		 *plprob_other = lprob_other;
 	   if (x1i) {
   		lprob1 = *plprob;
			lprob0 = lprob_other;
		 }
		 else {
		  lprob1 = lprob_other;
		  lprob0 = *plprob;
		  }

	 }
   f = exp(lprob1 - lprob0) * probx1i / (1.0 - probx1i);
   return  f  ;
}

state_t upd_state_and_S2(int npatch, int i, double probti, state_t xtiold,\
      double *St, double *ADai, \
			double *plprob, double lprob_other, double trprobti, double *newtrprobti, int *state_change)
/*speedy form of upd_state_and_S using and updating plprob
 state_change counts the number of state changes */
/*  draws a new state for patch i from a given probability and
    updates the connectivity vector St for the corresponding time  t
    from the matrix ADai ( Areaj, distances Di from patch i and alpha )
*/
{
  int j;
  state_t xtinew;

	# if ZEROONE
  	xtinew = (probti >urand(&seed) );
	#else
		xtinew = probti;
	# endif

	if (xtinew) *newtrprobti = trprobti; else *newtrprobti = 1 - trprobti;

  if (xtinew != xtiold){
		(*state_change)++;
		*plprob = lprob_other;

       
    for (j=1; j<=npatch; j++){
    /* update S from dis*/
	    # if ZEROONE
          if (xtinew) St[j] +=  ADai[j]; else St[j] -=  ADai[j];
 	    #else
          St[j] +=  ADai[j] * (xtinew - xtiold);
    	# endif
    }
    
  }
  return xtinew;
}


double Gibbs_patches2(long nsubiter, int nyear, int npatch, \
 state_t **Xs, int L, int StartPattern, int **ti_missinglist, double **S, double **ADa,\
 double *A, double **ptheta, int npar, double **X, int *nxpar, double **trprob,\
 double *pEloglik,double *pE2loglik, int startyear, int endyear)
/*speedy form of Gibbs_patches saving and re_using info of log(prob(Xt2 | Xt1})
 in the local variables lprob and lprob_other */
/* Use Gibbs sampling to update all missing patches from pretime -L to last data year nyear

   state_t  **Xs;   states with front times, data and imputed missing data 
   double *A,       vector of Areas  
	        **S,      times x patches matrix of connectivities
					**ADA,    patches x patches matrix of distances with elements A[j]^b*exp(-D[i][j]/alpha)   
          **X,      patches x nvars matrix of extra explanatory variables  
          **ptheta, pointers to rows of the times x npar matrix of parameter values  
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram

Returns the log likelihood of last iteration 
Eloglik is the average loglik
E2loglik is the estimated variance of the average likelihood, var(loglik)/n ,
		 with n the number of evaluations

Note:
	 **trprob is a times x npatch matrix with transition probabilities,
	 calculated in Gibbs_patches2 for the missing data only!!
	 
startyear and endyear determine the year over which the likelihood is calculated:

   if (startyear == 1 and endyear == nyear ) { usage of turnover information only	 }
   if (startyear == -L+1 and endyear == nyear ) { usage of spatial + turnover information 	 }
	  
*/
{
 int i,k, kk, t,   state_change = 0;
 long iter, iterEM=0;
 state_t stot;
 double probti, f, lprob, lprob_other, trprobti, loglik, 
	 Eloglik =0, /* average log likelihood if Stochastic EM*/
	 E2loglik =0;  /* ss of log likelihood if Stochastic EM*/
  
		time__2 = -10000;

		if (ti_missinglist[1][1]==INT_MAX) {
			/* no missings in the data. 
			Calculate once the transition probabities of data years, except the first year */
	 		calc_trprob(npatch, 2, nyear, Xs, S, A, ptheta, npar, X, nxpar, trprob);
		}

	for (iter = 1; iter<=nsubiter ; iter++ ) {
		
		/* All transition probabilities for the pretime years are calculated 
					and stored in the updating process.*/


			if (StartPattern >=0) {
	 /* update patches at pretime -L*/
		    t = -L;
		    for (i=1;i<=npatch ;i++ ){
		        trprob[t][i] = ptrans( 1, Xs[t+1][i], S[t+1][i], A[i], ptheta[-L], npar, X[i], nxpar);
		        #if !ZEROONE
		           Xs[t][i] = trprob[t][i];
		        #endif
		    }
		   #if   ZEROONE
		    do {
		      stot = 0;
		   	  for (i=1;i<=npatch ;i++ ){
		        Xs[t][i] = (trprob[t][i]>urand(&seed));
		        stot +=  Xs[t][i];
		       }
		    } while (stot < 1);
		    #endif

		  /*calculate connectivity for time t = -L*/
		    calc_S2( S[-L], npatch , Xs[-L], ADa);
			} /* if StartPattern */

	 /* update patches at pretimes from -L+1 to 0*/

	 for (t=-L+1; t<=0; t++){
	    for (i=1; i<=npatch; i++){
	      f = c_odds2(npatch, i, t, Xs[t-1][i], Xs[t], Xs[t+1],\
	      S[t-1][i], S[t], A, ADa[i], ptheta[t], ptheta[t+1], npar, X, nxpar,\
				&lprob, &lprob_other, &trprobti);
	      probti= f/(1+f);
	      Xs[t][i] = upd_state_and_S2(npatch, i, probti, Xs[t][i], S[t], ADa[i], \
	        &lprob, lprob_other, trprobti, &(trprob[t][i]), &state_change);
# if DEBUG==100
			if (t==0)fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
	    }
	 }


	/* update missings in all data year except last */

		for (k=1;(t = ti_missinglist[k][1])< nyear ;  k++ ) {
	  /*t = ti_missinglist[k][1];*/
			i = ti_missinglist[k][2];
	    f = c_odds2(npatch, i, t, Xs[t-1][i], Xs[t], Xs[t+1],\
	       S[t-1][i], S[t], A, ADa[i], ptheta[t], ptheta[t+1], npar, X, nxpar,\
				&lprob, &lprob_other, &trprobti);
	    probti= f/(1+f);
	    Xs[t][i] = upd_state_and_S2(npatch, i, probti, Xs[t][i], S[t], ADa[i], \
	       &lprob, lprob_other, trprobti, &(trprob[t][i]), &state_change);
# if DEBUG==100
			  fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
		}

	/* update missings in last data year */
		for (kk= k ;(t = ti_missinglist[kk][1])== nyear ;  kk++ ) {
	/*   t = ti_missinglist[k][1];      */
			i = ti_missinglist[kk][2];
	    probti = ptrans( 1, Xs[t-1][i], S[t-1][i], A[i], ptheta[t], npar, X[i], nxpar);
	    Xs[t][i] = upd_state_and_S2(npatch, i, probti, Xs[t][i], S[t], ADa[i], \
	       &lprob, lprob_other, probti, &(trprob[t][i]), &state_change);
# if DEBUG==100
			  fprintf(mon, " year %d patch %d odds %.2lf prob %.3lf\n", t, i, f, probti);
# endif
		}

		/*  The transition probabilities in the data themselves may have changed 
			 if the states at pretime 0 changed or if there are missings in the data  */

		if (ti_missinglist[1][1]==INT_MAX) {
			/* no missings in the data, but
			the transition probabities of first year's data may have changed if
			the states at pretime 0 changed, thus calculate from scratch */
	 		calc_trprob(npatch, 1, 1, Xs, S, A, ptheta, npar, X, nxpar, trprob);
		}
		else {
			/*missings in the data years */
 			calc_trprob(npatch, 1, nyear, Xs, S, A, ptheta, npar, X, nxpar, trprob);
		}

		if (Stochastic_EM !=0) {
			/* calculate average loglikelihood Eloglik, after 10% of iterations */
			if (iter > 0.1 * nsubiter) {
				iterEM++;
	/*
		fprintf(out," Xs 1\n");
		fprintfimatrix(out, Xs, -L, nyear, 1, npatch, " %2d " );

		fprintf(out," trprob 1\n");
		fprintfdmatrix(out, trprob, -L+1, nyear, 1, npatch, " %10.3lf " );
		
  	calc_trprob(npatch, -L+1, 0, Xs, S, A, ptheta, npar, X, nxpar, trprob);
    fprintf(out," trprob 2\n");
 		fprintfdmatrix(out, trprob, -L+1, nyear, 1, npatch, " %10.3lf " );

*/
			 loglik = floglik2(npatch,startyear, endyear, trprob);
			 update_mean_sumss(iterEM, loglik, &Eloglik, &E2loglik);
 			}

		}

	} /*  end iter loop*/

 		/*calc_trprob(npatch, -L+1, 0, Xs, S, A, ptheta, npar, X, nxpar, trprob);*/

		# if DEBUG == 103

		fprintf(out," changes in state = %d \n", state_change);
		fprintf(out," trprob in Gibb2\n", state_change);
		fprintfdmatrix(out, trprob, -L+1, nyear, 1, npatch, " %10.3lf " );
		# endif

    loglik = floglik2(npatch, startyear, endyear, trprob);
		if (Stochastic_EM && iterEM>1 )	E2loglik /= iterEM*(iterEM-1);
		*pEloglik =  Eloglik;
		*pE2loglik = E2loglik;
		


	return loglik;
}

double floglik2(int npatch, int startyear, int endyear, double **trprob)
/* loglik calculates the loglikelihood of the model 
		from years startyear to and including endyear
		using already transition probabilities in trprob

   double  **trprob,times x npatch matrix of transition probabilities
The double ** pointers involving times have fist index running from  -L to nyears
 in their definition in the mainprogram
startyear and endyear must therefore lie in the range -L , nyear

If DEBUG
output: 
trprob transition probabilities for t = startyear to endyear (all npatch patches)
			
*/
{ 
	int t,i;
	double llik;

	llik = 0;

	  for (t =startyear;t<=endyear ;t++ ) 
   		for (i=1;i<=npatch ;i++ ) llik += log(trprob[t][i]);			
			

	 return llik;
}
void init_Xs_S1(int nyear, int npatch, int **idata, int **ti_missinglist, int nmissing, \
 state_t **Xs, int L, int StartPattern, double **S, double **ADa,\
 double *A, double *theta_ini, int npar, double **X, int *nxpar, double f_ini)
 /*initialize
   - Xs (the states) from data (idata) and imputes the missings by simple imputation
	 					using the initial model parameters  theta_ini
	 - S  (the connectivities) from Xs, A, D (distances) and alfa
	 Xs and S are (-L...nyear) by (1...npatch)  
*/	
{
	int t0,t,i,k;
	state_t stot;
	double *trprob;

  trprob    = dvector(1,npatch);

  for (t =1;t<=nyear ;t++ )	{for (i=1;i<=npatch ;i++ )	Xs[t][i] =idata[t][i];}


	for (k=1;k<=nmissing;  k++ ) {
/* fill missings in all data year */
    t = ti_missinglist[k][1];
		i = ti_missinglist[k][2];
		# if ZEROONE
		Xs[t][i]= (urand(&seed) < f_ini);
		#else
		Xs[t][i]= urand(&seed);
		# endif
	}

/* impute missings for preyears 0 to -L*/

  for (t =0;t>=-L ;t-- ) {
    calc_S2( S[t+1], npatch , Xs[t+1], ADa);
   	for (i=1;i<=npatch ;i++ ){
        trprob[i] = ptrans( 1, Xs[t+1][i], S[t+1][i], A[i], theta_ini, npar, X[i], nxpar);
        #if !ZEROONE
           Xs[t][i] = trprob[t][i];
        #endif
    }
   #if   ZEROONE
    do {
      stot = 0;
   	  for (i=1;i<=npatch ;i++ ){
        Xs[t][i] = (trprob[i]>urand(&seed));
        stot +=  Xs[t][i];
       }
    } while (stot < npatch/20.0);
   #endif
 }
	 if ((StartPattern == 111)||(StartPattern == -111) ) {
	 	
	 		 	/*  start with ones and one step of extinctions */
				t0 = -(L+1);
				t = -L;
				for (i=1;i<=npatch ;i++ )  Xs[t0][i] = 1; 

        calc_S2( S[t0], npatch , Xs[t0], ADa);
			  for (i=1;i<=npatch ;i++ ){
			      trprob[i] = ptrans( 1, Xs[t0][i], S[t0][i], A[i], theta_ini, npar, X[i], nxpar);
			      #if !ZEROONE
			          Xs[t][i] = trprob[t][i];
			      #endif
			  }
			  #if   ZEROONE
			  do {
			    stot = 0;
			   	for (i=1;i<=npatch ;i++ ){
			      Xs[t][i] = (trprob[i]>urand(&seed));
			      stot +=  Xs[t][i];
			      }
			  } while (stot ==0);
			  #endif


    

  }
	/* calculate connectivity S[t] for year t (t= -L, 2 ... nyear) */

  calc_S2( S[-L], npatch , Xs[-L], ADa);
  for (t =2;t<=nyear ;t++ ) calc_S2( S[t], npatch , Xs[t], ADa);
  
  fprintf(out," connect S times x npatch\n");
  fprintfdmatrix(out,S,-L,nyear,1,npatch, "%10.2lf");

  free_dvector(trprob,1,npatch);
}

# endif  /*QUICK == 1*/

