
/* functions fmetapop of updating per column (per year) */ 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <limits.h>
#include <nrutil.h>
#include <nr.h>


#include  "fmetapop.h" 
#include  "MC_patches.h"

extern double logEPS, log1_EPS;
 

double logptrans(state_t x1i, state_t x0i, double S0, double logA, double logE1_Ei[], 
              double theta0[], int npar, double X[], int *nxpar)
/* logptrans calculates the logtransition probability
   to state x1i of patch i at time 1,
   for a given patch (i) given state x01 at time 0 (of patch 1) 
   and the connectivity value S0 of the patch from time 0, its area A,
   extra explanatory variables in X[], 
   and the npar-vector of parametervalues  theta0 at time 0
	 nxpar[0] = number of X variables for E model
	 nxpar[1] = number of X variables for C model
   x1i = state at time 1 of patch i
   x0i = state at time 0 of patch i
   The state can be  0 or i or a value between 0 and 1
	 if !regional stochast 2-vector logE1_E is used with on
	 Input: logE1_Ei[0] = log(E) and  logE1_Ei[1] = log(1-E)

	 Routine is used if QUICK >=2
   
   */
{
 #define ONE  1.0E0
 #define ZERO 0.0E0
 #define EPS0 1.0E-20
 state_t EMPTY=0;
 int j, beginE, beginC;
 double logprob, logE, log1_C, loge, x, logy, z, lp;
 /*
  loge = theta0[1];
  x = theta0[2];
  logy = theta0[3];
  z = theta0[4];
  alpha = theta0[5]
	*/
	 /* indexes in theta for the X variables for E model and C model */
	 beginE = npar - nxpar[0] - nxpar[1];
	 beginC = npar - nxpar[1];
	 /*  state_t == int */
      if (x0i) {
				/* extinction probabilities*/
			  #if REGIONAL_STOCHAST
					 loge = theta0[1];
					 x = theta0[2];
				   lp = ZERO;
					 for (j= 1;j<= nxpar[0] ;j++ )  lp +=  theta0[beginE+j]*X[j];
					 logE = loge - x * logA + lp;  
				   logE = DMIN(logE, log1_EPS);
				   #if WITH_RESCUE_EFFECT
	      		  logy = theta0[3];
			    		z = theta0[4];
						  lp = ZERO;
						  for (j= 1;j<= nxpar[1] ;j++ )  lp +=  theta0[beginC+j]*X[nxpar[0]+j];
		  			  if (S0 <= EPS0) log1_C = log1_EPS; else log1_C = -log(ONE + exp(logy+z*log(S0)+lp ));
				      logE += log1_C;
				   #endif
				   if (x1i) logprob = log(ONE - exp(logE));  else logprob = logE;
	 			#else /* E constant in time */
				   #if WITH_RESCUE_EFFECT
  					 logE = logE1_Ei[EMPTY];
	      		 logy = theta0[3];
			    	 z = theta0[4];
						 lp = ZERO;
						 for (j= 1;j<= nxpar[1] ;j++ )  lp +=  theta0[beginC+j]*X[nxpar[0]+j];
		  			 if (S0 <= EPS0) log1_C = log1_EPS; else log1_C = -log(ONE + exp(logy+z*log(S0)+lp ));
				     logE += log1_C;
   				   if (x1i) logprob = log(ONE - exp(logE));  else logprob = logE;
					 #else
  					 logprob = logE1_Ei[x1i];
				   #endif
				# endif
      }
      else {
				 /* colonization probabilities*/
  		   logy = theta0[3];
				 z = theta0[4];
	       if (x1i){ /*logC*/
					 if (S0 <= EPS0) logprob = logEPS;
				   else {
						lp = ZERO;
						for (j= 1;j<= nxpar[1] ;j++ )  lp +=  theta0[beginC+j]* X[nxpar[0]+j];
            logprob = -log( ONE + exp(-(logy+z*log(S0)+lp)  ));		
			     /* logprob = DMAX(logprob, logEPS);	/*  close to 0 */
					 } 
 
				 }
				 else { /*log1_C*/
					 if (S0 <= EPS0) logprob = log1_EPS;
				   else {
						lp = ZERO;
						for (j= 1;j<= nxpar[1] ;j++ )  lp +=  theta0[beginC+j]* X[nxpar[0]+j];
					  logprob = -log( ONE + exp(logy+z*log(S0)+lp ));
	    		  /* logprob = DMIN(logprob, log1_EPS);	 /*	 log(1-C) close to 1; C close to 0*/
					 } 
				 }
       }
			  /* avoid numerical trouble with prob = 0 and 1 - prob = 0  */
			  logprob = DMAX(logprob, logEPS);
			  logprob = DMIN(logprob, log1_EPS);
	      
  return logprob;
}

void ftrans_theta2(double *theta, double *trans_theta, int npar, double *trans_par)
/* transforms the parameter vector theta
					 to the paramater vector trans_theta on the linkscale*/
{
	int k;

	/* initialize with no transformation */
	for (k=1;k<=npar ;k++ ) trans_theta[k] = theta[k];


	 /* put here transformations*/

	 /*  parameter theta[1] = loge = log(e)   */
	 /* apply a centring to logA, i.e. trans_par[1] = mean(logA)
	   log E = loge                  - x * logA =
		 log E = loge - x * mean(logA)  - x * (logA - mean(logA))
		 thus
	       trans_theta1 =  log (e) - x * mean(log(A)) */
	 trans_theta[1] = theta[1] - theta[2]*trans_par[1];

	 /*  parameter theta[3] = -log(y2)*/
	 /* apply a centring to logS, i.e. trans_par[3] = mean(logS)
	   logit(C) = c + z logS = c + z * mean(logs) + z * (logS - mean(logS) 
		 thus transc = c + zlogS with c = -log(y2) 
	       trans_theta3 =  -log (y2) + z * mean(log(S)) */
	 trans_theta[3] = theta[3] + theta[4]*trans_par[3];

	 trans_theta[2] = log(theta[2]);
	 trans_theta[4] = log(theta[4]);
	 trans_theta[5] = log(theta[5]);
	 trans_theta[6] = log(theta[6]);



}

void fbacktrans_theta2(double *theta, double *trans_theta, int npar, double *trans_par)
/* backtransforms the parameter vector trans_theta on the  linkscale
									 to paramater vector theta 			 on the original scale */
{
	int k;

	/* initialize with no transformation */
	for (k=1;k<=npar ;k++ ) theta[k] = trans_theta[k];


	 /*  parameter loge*/
	
	 /*      theta1 =   trans_theta1 + x * mean(log(A))  */
	 theta[1] = trans_theta[1] + theta[2]*trans_par[1];

	 /*  parameter logy*/
	 /*      theta3 =     trans_theta3 - z * mean(log(S))  */
	 theta[3] = trans_theta[3] - theta[4]*trans_par[3];

	 theta[2] = exp(trans_theta[2]);
	 theta[4] = exp(trans_theta[4]);
	 theta[5] = exp(trans_theta[5]);
	 theta[6] = exp(trans_theta[6]);


}

void fbacktrans_ptheta2(double **ptheta, double **trans_ptheta, int npar, double *trans_par)
/* backtransforms the parameter matrix trans_ptheta on the  linkscale
									 to paramater matrix ptheta 			 on the original scale
 taking care about REGIONAL_STOCHAST 0/1
									  
									 */
{
#if !REGIONAL_STOCHAST
   fbacktrans_theta2(ptheta[0], trans_ptheta[0], npar, trans_par);
#else
  for (t=-L;t<=nyear ;t++ ) fbacktrans_theta2(ptheta[t], trans_ptheta[t], npar, trans_par); 
  
#endif
}

void	myfwrite(FILE *out, long iter, int lag, double loglik0, double logprior0, double **theta, double **trans_theta, int npar, double accept_p) 
/* writes every lag-th metropolis hastings sample to a file */
{
	int k;
	#if QUICK <= 1
	if (iter ==1) {
			fprintf(out, "       iter     loglik   logprior    log e(t)    x           logy         z             alpha"
			                                                       "     e           y2     p_accept_c \n");
		}
	#else
	if (iter ==1) {
		if (npar == 0) {
			fprintf(out, "If EM  iter    Eloglik  sd_loglik  p_accept_c \n");
	  }
		else {
/*			fprintf(out, "       iter     loglik   logprior   log e*(t)   x         -logy*         z             alpha"*/
			  fprintf(out, "       iter     loglik   logprior  log e     logx         -logy      logz"
"       logalpha      log b     p_accept_c          q1        q2\n");
	  }
	}		

	#endif

	if (iter%lag ==0){
			fprintf(out,  " %10ld %10g %10g", iter, loglik0, logprior0);
#if !REGIONAL_STOCHAST
			for (k=1;k<=6 ; k++) fprintf(out,  " %10g ", trans_theta[0][k] );
		 /*
		 	if (npar>0) {
			 fprintf(out,  " %10g ", theta[0][1] );
			 fprintf(out,  " %10g ", theta[0][3] );
			}
			*/
			/*
			for (k=1;k<=npar ; k++) fprintf(out,  "%10g  ", theta[0][k] );
			*/
#else
  for (t=-L;t<=nyear ;t++ ){
			fprintf(out,  "\n %4d\n ", t);
			for (k=1;k<=npar ; k++) fprintf(out,  "%10g ", trans_theta[t][k] );
	
	} 
  
#endif
	fprintf(out,  "%10g", accept_p);
			for (k=7;k<=npar ; k++) fprintf(out,  " %10g ", trans_theta[0][k] );
	fprintf(out,  "\n");

 /* fflush(out);	*/
	}
												

	
	/*
	fprintf(out,  " ", );
	*/
}


void	myfwrite2(FILE *out, long iter, int lag, state_t **Xs, int **ti_missinglist,
 int nyear, int indexnyear) 
/* writes every lag-th metropolis hastings sample of missing Xs in endyear to a file */
{
	int i,j,k;
	if (iter ==1) {
/*			fprintf(out, "       iter     loglik   logprior   log e*(t)   x         -logy*         z             alpha"*/
			  if (ti_missinglist[indexnyear][1]==nyear){
					fprintf(out, " table of simulated missing states in last year \n");
					for (j=indexnyear; ti_missinglist[j][1]==nyear ;j++ ) {
						 fprintf(out, " %2d", ti_missinglist[j][2]);
					}
		  		fprintf(out,  "\n" );
		  	}
	}		



	if (iter%lag ==0){
				for (j=indexnyear; ti_missinglist[j][1]==nyear ;j++ ) {
					i= ti_missinglist[j][2];
					 fprintf(out, " %2d", Xs[nyear][i]);
				}
	  		if (ti_missinglist[indexnyear][1]==nyear) fprintf(out,  "\n" );

 /* fflush(out);	*/
	}
												

	
}



